/**************************************************************************//**
 *
 * Copyright (C) 2011 Infineon Technologies AG. All rights reserved.
 *
 * Infineon Technologies AG (Infineon) is supplying this software for use with 
 * Infineon's microcontrollers.  
 * This file can be freely distributed within development tools that are 
 * supporting such microcontrollers. 
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, 
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
********************************************************************************
**                                                                            **
**                                                                            **
** PLATFORM : Infineon XMC4000 Series   			                          **
**                                                                            **
** COMPILER : Compiler Independent                                            **
**                                                                            **
** AUTHOR   : App Developer                                                   **
**                                                                            **
** MAY BE CHANGED BY USER [yes/No]: Yes                                       **
**                                                                            **
** MODIFICATION DATE : Dec 12, 2012                                           **
**                                                                            **
*******************************************************************************/

/*******************************************************************************
**                       Author(s) Identity                                   **
********************************************************************************
**                                                                            **
** Initials     Name                                                          **
** ---------------------------------------------------------------------------**
** NPJ        App Developer                                                   **
*******************************************************************************/
/* @file DBG002_Private.h
 *
 * @brief  Private Header file for DebugLog.
 *
 */

#ifndef DBG002_PRIVATE_H
#define DBG002_PRIVATE_H

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************************
**                                  File Include                              **
*******************************************************************************/
/**
 * @cond INTERNAL_DOCS
 */
#include <DAVE3.h>

/*******************************************************************************
**                      Global Macro Definitions                              **
*******************************************************************************/

/**
 * @addtogroup DBG002_privateparam
 * @{
 */

/** Maximum number of APPs */
#define DBG002_MAX_NO_OF_APP                        0xFF

/** Maximum length of Debug Message */
#define DBG002_MAX_MSG_LEN                          0xFF

/** Message Length of the message Set Dynamic Filter */
#define DBG002_MSGLEN_CMD_SETDYNAMICFILTER            34

/** Message Length of the message change Severity */
#define DBG002_MSGLEN_CMD_CHANGESEVERITY               3


#define DBG002_MSG_LEN                       DBG002_MAX_MSG_LEN


/*******************************************************************************
**                      Global Type Definitions                               **
*******************************************************************************/

/**
 * @brief Enumeration for type of commands from HOst
 */

typedef enum DBG002_CommandIDType
{
  /** Command to change the dynamic filter for the channel */
  DBG002_SET_DYNAMIC_FILTER,
  /** Messages with severity below CMDID_Severity will be stored in circular buffer. Once the message severity is = to OR
   * greater than the set severity messages are dumped on debug log */
  DBG002_CHANGE_SEVERTIY,
  /** Set value of memory location @ address, with size (8,16,32 bits)*/
  DBG002_SET_LOCATION = 4,
  /** Send back to the host the value of size (8,16,32bits) @ address.*/
  DBG002_GET_LOCATION = 5,
  /** Send back to the host the value of size (8,16,32bits) @ address.*/
  DBG002_GET_INFO = 7,
  /** XSPY related commands*/
  DBG002_XSPY = 0xFF
} DBG002_CommandIDType;

/**
 * @brief Format of debug message when it is added to buffer or sent to host
 */
typedef struct DBG002_LoggingMessageFormatType
{
  /** Start byte for debuglog message */
  uint8_t StartByte[2];
  /** wrap around counter for loss of message detection */
  uint8_t Counter;
  /** Time at which message logged */
  uint8_t TimeStamp[3];
  /** Severity Level */
  uint8_t SeverityLevel;
  /** Unique ID for Application */
  uint8_t GroupID;
  /** Error ID */
  uint8_t MessageID;
  /** Length of the error message */
  uint8_t MessageLen;
  /** Message */
  uint8_t Message[DBG002_MSG_LEN];
  /** Counter 2*/
  uint8_t Counter2;
  /** Buffer data status */
  uint8_t isNew;
  /** Buffer address for stream data */
  DBG002_PingPongBufType* StreamBufPtr;
} DBG002_LoggingMessageFormatType;

/**
 * @}
 */

/*******************************************************************************
**                       FUNCTION PROTOTYPES                                  **
*******************************************************************************/

/** @addtogroup DBG002_privatefunc
 * @{
 */

/**
 * @brief       This function initializes UART channel configured
 *
 * @param[in]   Handle: Uart input handle
 *
 * @return      none
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 *
 */
void DBG002_lUARTInit(void* Handle);



/**
 * @brief       This function writes the data to Usic fifo buffer for
 *              transmisssion
 *
 *
 * @param[in]   Handle1: Uart input handle
 * @param[in]   Len: Length of message
 * @param[in]   Buffer: Pointer to buffer 
 *
 * @return      none
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 *
 */
status_t DBG002_UARTWrite(void* Handle1, uint32_t Len, uint8_t* Buffer);

/**
 * @brief       Parses the command received from Host. This function will call
 * 			    the call back function registered depending on the group ID received
 * 			    from the host. Target won't receive any other command from host until
 * 			    the previous command is processed
 *
 *
 * @param[in]   PtrCmdBuffer Pointer to the  command buffer
 *
 * @return      status_t<BR>
 * 				DAVEApp_SUCCESS: If successfull <BR>
 * 				Others: Status returned from registered call back function.<BR>
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 */
status_t DBG002_lProcessCommand(DBG002_CommandFromHostType* PtrCmdBuffer);

/**
 * @brief       This modifies the dynamic filter parameters. The parameters are
 *              received from the Host.
 *
 * @param[in]   CmdFromHost Pointer to the command sent by host
 *
 * @return      status_t
 * 				DBG002_ERROR: if input command is wrong or filter type is
 * 							  set to Dynamic
 * 				DAVEApp_SUCCESS: if successful
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 *
 */
status_t DBG002_lSetDynamicFilter(const DBG002_CommandFromHostType* CmdFromHost);


/**
 * @brief       This is the debuglog apps callback.
 *              This is registered with debuglog app.
 *
 * @param[in]   CmdFrmHost Command sent by the Host
 *
 * @return      status_t
 * 				DBG002_ERROR: if input command is wrong;
 * 				DBG002_SETLOCATION_FAILED: if set location command processing failed
 * 				DBG002_GETGROUPID_FAILED: if get group ID command failed
 * 				DBG002_GETLOCATION_FAILED: if get location command failed
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 *
 */
status_t DBG002_lCallback(void* CommandPtr,uint32_t GroupID,DBG002_XSpyDataType* XspyAppPtr);

/**
 * @brief       Adds the debug Message to the Circular Buffer of appropriate
 *              channel.
 *
 * @param[in]   Severity_Level Message Severity Level
 * @param[in]   GroupID        Application ID
 * @param[in]   MessageID      Message / Error ID
 * @param[in]   Length         Length of the Message
 * @param[in]   Message        Pointer to the Message
 *
 * @return      none
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 *
 */
void DBG002_lAddMsgToBuffer
(
  uint32_t Severity_Level,
  uint8_t GroupID,
  uint8_t MessageID,
  uint8_t Length,
  uint8_t *Message
);

/**
 * @brief       Sends the message to Host by calling appropriate API of the
 *              peripheral interface
 *
 * @return       None
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 *
 */
void DBG002_SendMessageToHost(void);
/*
 * This is call back function registered by the Api DBG002_XspyInit() for
 * processing xSPY commands recived from host.
 * */
/**
 * @brief  This is call back function registered by the Api DBG002_XspyInit() for
 *         processing xSPY commands recived from host.
 *
 * @param[in]   CommandPtr pointer to received command buffer
 * @param[in]   GroupID        Application ID
 * @param[in]   XspyDataPtr   pointer to xspy data structure

 *
 * @return     None
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 *
 */
status_t DBG002_lXspyCmdHandler
(
  void* CommandPtr,
  uint32_t GroupID,
  DBG002_XSpyDataType *XspyDataPtr
);


/**
 * @brief       This API is used for static filtering of messages.
 *
 * @param[in]   Severity_Level Severity level of the Message
 * @param[in]   GroupID        Application ID
 * @param[in]   MessageID      Message / Error ID
 * @param[in]   Length         Length of the Message
 * @param[in]   Message        Pointer to the Message
 *
 * @return     None
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 *
 */
void DBG002_lStaticFilter
(
  uint32_t Severity_Level,
  uint8_t GroupID,
  uint8_t MessageID,
  uint8_t Length,
  const uint8_t *Message
);

/**
 * @brief       This API is used for dynamic filtering of messages.
 *
 * @param[in]   Severity_Level Severity level of the MEssage
 * @param[in]   GroupID        Application ID
 * @param[in]   MessageID      Message / Error ID
 * @param[in]   Length         Length of the Message
 * @param[in]   Message        Pointer to the Message
 *
 * @return     None
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 *
 */
void DBG002_lDynamicFilter
(
  uint32_t Severity_Level,
  uint8_t GroupID,
  uint8_t MessageID,
  uint8_t Length,
  uint8_t *Message
);

/**
 * @brief    Function which implement Fletcher32 chacksum
 *
 * @param[in]   data: pointer to input buffer
 * @param[in]   words: No of words
 *
 * @return     uint32_t calculated checksum value
 *
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 *
 */
uint32_t DBG002_lFletcher32( uint16_t const *data, uint32_t words );


/**
 * @brief    Function to check if received checksum and calculated checksum are same.
 *
 *
 * @return     uint8_t
 *             TRUE : if checksum received is correct
 *             FALSE: if checksum received is wrong
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 *
 */
uint8_t DBG002_lIsCheckSumOk(void);

/**
 * @}
 */

/**
 * @endcond
 */
 
#ifdef __cplusplus
}
#endif 

#endif /* DBG002_PRIVATE_H*/


