/*******************************************************************************
 *
 * Copyright (C) 2011 Infineon Technologies AG. All rights reserved.
 *
 * Infineon Technologies AG (Infineon) is supplying this software for use with
 * Infineon's microcontrollers.
 * This file can be freely distributed within development tools that are
 * supporting such microcontrollers.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE.
 * INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL,
 * OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
********************************************************************************
**                                                                            **
**                                                                            **
** PLATFORM : Infineon XMC4000 Series                                         **
**                                                                            **
** COMPILER : Compiler Independent                                            **
**                                                                            **
** AUTHOR : App Developer                                                     **
**                                                                            **
** MAY BE CHANGED BY USER [yes/no]: Yes                                       **
**                                                                            **
** MODIFICATION DATE : Dec 13, 2012                                           **
**                                                                            **
*******************************************************************************/

/*******************************************************************************
**                       Author(s) Identity                                   **
********************************************************************************
**                                                                            **
** Initials    Name                                                           **
** ---------------------------------------------------------------------------**
** NPJ          App Developer                                                  **
*******************************************************************************/


#ifndef DBG002_XSPY_H_
#define DBG002_XSPY_H_

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************************
 *                                INCLUDE FILES                               **
 ******************************************************************************/
#include <DAVE3.h>

/**
 * @ingroup DBG002_publicparam
 * @{
 */
/*******************************************************************************
**                      Global Macro Definitions                              **
*******************************************************************************/
#define DBG002_STREAM_BUFF_LENGTH          255

#define DATA_MASK 0xFFU
/**
 * @brief XSpy message Length
 */
#define DBG002_XSPY_MESSAGE                 (DBG002_MSG_LEN - 8)
/**
 * @brief XSpy Command Length
 */
#define DBG002_XSPY_COMMAND                 (DBG002_MSG_LEN - 2)

#define MESSAGE_ID_XSPY                     0xFF
#define MAX_STRING_LENGTH 60U

/*******************************************************************************
 *                                ENUMERATIONS                                **
******************************************************************************/
/**
 * @brief Enumeration for Command buffer status
 */
typedef enum DBG002_CmdbufferStatusType
{
  /** Command buffer is ready to accept */
  DBG002_BUF_READY,
  /** Previous command received is not processed */
  DBG002_BUF_BUSY
} DBG002_CmdbufferStatusType;

/**
 * @brief XSPY message ID
 */
typedef enum DBG002_XSpyMessageIdType
{
  /** MessageId for XSpy GetLocation*/
  DBG002_XSPYGetLocation,
  /** MessageId for sending variable table*/
  DBG002_XSPYSendVariableTable,
  /** MessageId for sending variable table*/
  DBG002_XSPYSendStreamData,
} DBG002_XSpyMessageIdType;

/**
 * @brief XSPY Command ID
 */
typedef enum DBG002_XSpyCommandIdType
{
  /** CommandId for XSPY Set stream data  */
  DBG002_XSPYSetStreamData,
  /** CommandId for XSPY Set sync Command  */
  DBG002_XSPYSetSyncCmd,
  /** CommandId for XSPY Execute sync Command  */
  DBG002_XSPYExecuteSyncCmd,
  /** CommandId for get variable table  */
  DBG002_XSPYGetVariableTable,

}DBG002_XSpyCommandIdType;


/*******************************************************************************
 *                                STRUCTURES                                  **
 ******************************************************************************/
/**
 * @brief Format of Command sent by the Host
 */
typedef struct DBG002_CommandFromHostType
{
  /** Group ID: ID of the application for which the command is intended for */
  uint8_t GroupID;
  /** Counter: wrap around counter used for detecting the loss of messages */
  uint8_t Counter;
  /** Message Length: Length of the command message which will follow */
  uint8_t MessageLen;
  /** Message: Actual Message (LSByte First) */
  uint8_t Message[249];
  /** CheckSum value */
  uint8_t CheckSum[4];
} DBG002_CommandFromHostType;
/**
 * @brief Data structure to hold pinp pong buffer
 * information for stream data
 */
typedef struct DBG002_PingPongBufType
{
	uint8_t Buffer[256];
	uint8_t BufferStatus;
}DBG002_PingPongBufType;


/**
 * @brief This structure type defines the variable description . Each App
 * can create an array of variable description of this type and can be registered in
 * API DBG002_XspyInit(). This variable description table will be send to host when host
 * sends the get variable command.
 */
typedef struct DBG002_VariableDescriptionType
{
  /** Name of a variable and additional information about the app can be stored 
   *  in ':'. Example: for motor variable "name of variable(Speed):scale:unit" =>
   *  "Speed:4.1:rpm" => 
   */
  uint8_t Name[60];
/** size of a variable in bytes*/
  uint8_t Size;
/** Address of a variable*/
  uint32_t Address;
} DBG002_VariableDescriptionType;

/**
 * @brief Monitored App command structure
 */
typedef struct DBG002_AppCommandStructureType{
  /** Command Address*/
  uint32_t Address;
  /** data size*/
  uint8_t Size;
  /** command */
  union DBG002_Command_t
  {
    /** 8 bit value*/
    uint8_t Value8Bit;
    /** 16 bit value*/
    uint16_t Value16Bit;
    /** 32 bit value*/
    uint32_t Value32Bit;
    /** 64 bit value */
    double Value64Bit;
  } DBG002_Command;
} DBG002_AppCommandStructureType;


/**
 * @brief XSpy Data type. This structure object has to be created for each App to support
 * xSPY. This structure contains all the global variable information required to support xSPY 
 * for the App.
 */
typedef struct DBG002_XSpyDataType
{
  /** Configurable in debug_log app GUI up to 256*/
  DBG002_PingPongBufType StreamBuffer[2];
  /** current stream index no used by DBG002_XspyStreamData() Api */
  uint8_t * CurrentStreamBufferPtr;
  /**Address for which data needs to be streamed*/
  uint32_t StreamAddresses[ DBG002_MAX_STREAM_DATA];
  /**Corresponding size of data 1byte,2byte and 4 byte */
  uint8_t StreamSize[ DBG002_MAX_STREAM_DATA];
  /** XSPY_NUM_COMMANDS configurable in debug_log App GUI 
    This Data type contains the variable information of set
    sync command. App can read the address, size and value to
    be written to the variable. It is application responsibilty to
    set the corresponding value to the variable. App can check  
    SyncCommandsBufferReady to know whether sync command is 
    received*/
  DBG002_AppCommandStructureType SyncCommands[DBG002_NUM_OF_COMMANDS];
  /**Pointer to Variable description table*/
  DBG002_VariableDescriptionType *VarDescription;
  /**Number of elements in variable description table*/
  uint32_t NumberofVarDescription;
  /** Current index of stream Data */
  uint8_t CurrentStreamDataIndex;
  /** Size of data frame */
  uint8_t DataFrameSize;
  /**Current Command Index*/
  uint8_t CurrentCommandIndex;
  /**Gid of the App*/
  uint8_t Gid;
  /** Set this flag to TRUE when sync command buffer is ready for execution.
   *  This flag will be cleared when Execute sync command is processed*/
  volatile bool SyncCommandsBufferReady;
  /** Set this flag to TRUE when start stream command is received from host 
   * This flag will be cleared when target receives stop stream command from
   * host*/
  volatile bool StartStream;
  /** Flag to store the current ping pong buffer*/
  uint8_t PPBufferTracker;
} DBG002_XSpyDataType;

/**
 * @}
 */
 
 /*******************************************************************************
**                          FUNCTION PROTOTYPES                               **
*******************************************************************************/

/** @ingroup DBG002_apidoc
 * @{
 */


/** @ingroup DBG002_apidoc
 * @{
 */

/**
 * @brief       This API is used for XSpy data structure initilization . Each App can register
 * 				there variable description table by calling this Api. Call back function will also be 
 * 				registered for the specified group ID 
 *
 * @param[in]   *XspyDataPtr  This structure shall be part of the app instance handle. This structure
 *                         contains information data to be streamed. and buffer for storing the data
 * @param[in]   *VarDescriptorPtr  Pointer to the variable description table
 * @param[in]   NumberofVarDescriptor Number of variables in the descrition table
 * @param[in]   Gid    Group ID of the Application
 *
 * @return     status_t <BR>
               DAVEApp_SUCCESS: if init is successful<BR>
               DBG002_INVALID_PARAMETER: if input value is invalid<BR>
               
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 * <BR><P ALIGN="LEFT"><B>Example:</B>
 * @code
 * #include <DAVE3.h>      //Declarations from DAVE3 Code Generation
 *
 * // A few globals
 * uint8_t Speed;
 * uint16_t Cur;
 * uint32_t Voltage;
 * uint16_t Degree;
 * uint16_t Torque;
 * uint32_t Counter;
 * uint32_t Points;
 *
 * // Variable tables.
 * DBG002_VariableDescriptionType
 * Global1VarTable[5] = {
 *   XSPY_ADD_UINT8("Speed", Speed),
 * 	 XSPY_ADD_UINT16("Current", Cur),
 * 	 XSPY_ADD_UINT32("Voltage", Voltage),
 * 	 XSPY_ADD_UINT16("Degree", Degree),
 * 	 XSPY_ADD_UINT16("Torque", Torque)
 * };
 * 		                                        
 * DBG002_VariableDescriptionType
 * Global2VarTable[2] = {
 *   XSPY_ADD_UINT32("Counter", Counter),
 *   XSPY_ADD_UINT32("Points", Points)
 * };
 *
 * DBG002_XSpyDataType XpyApp1;
 * DBG002_XSpyDataType XpyApp2;
 * 
 * int main(void) {
 *   DAVE_Init();
 *   // Xspy init function
 *   DBG002_XspyInit(&XpyApp1, Global1VarTable, 5, DBG002_GID_XSPY1);
 *   // Xspy Init Function
 *   DBG002_XspyInit(&XpyApp2, Global2VarTable, 2, DBG002_GID_XSPY2);
 *   while(1) {
 * 		// Do nothing here
 *   }
 * }
 * @endcode<BR> </p> 
 */
status_t DBG002_XspyInit
(
  DBG002_XSpyDataType *XspyDataPtr,
  DBG002_VariableDescriptionType *VarDescriptorPtr,
  uint32_t NumberofVarDescriptor,
  uint8_t Gid
);
 
/**
 * @brief This API is used by monitored App for streaming data. If target receives
 * 		  start stream command, then App can use this function to stream data to
 * 		  host. User has to call this Api in application control loop to process
 * 		  stream data.
 *
 * @param[in]   XspyData  This structure shall be part of the app instance handle. This structure
 *                        contains information data to be streamed and buffer for storing the data    
 *                                      
 * @return     status_t <BR>
 *             DBG002_INVALID_PARAMETER: if input value is invalid<BR>
 *             DAVEApp_SUCCESS: if successfull<BR>
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 * <BR><P ALIGN="LEFT"><B>Example:</B>
 * @code 
 * #include <DAVE3.h>      //Declarations from DAVE3 Code Generation
 *
 * // A few globals
 * uint8_t Speed = 0x01;
 * uint16_t Cur = 0x0102;
 * uint32_t Voltage = 0xff22ff33;
 * uint16_t Degree = 0xff34;
 * uint16_t Torque = 0xab34;
 * 
 * // Variable tables.
 * DBG002_VariableDescriptionType
 * Global1VarTable[5] = {
 *   XSPY_ADD_UINT8("Speed", Speed),
 * 	 XSPY_ADD_UINT16("Current", Cur),
 * 	 XSPY_ADD_UINT32("Voltage", Voltage),
 * 	 XSPY_ADD_UINT16("Degree", Degree),
 * 	 XSPY_ADD_UINT16("Torque", Torque)
 * };
 * 
 * DBG002_XSpyDataType XpyApp;
 * 
 * int main(void) {
 *   DAVE_Init();
 *   // Xspy init function
 *   DBG002_XspyInit(&XpyApp, Global1VarTable, 5, DBG002_GID_XSPY1);
 *   while(1) {
 * 	   // Process command if received
 *     DBG002_XspyProcessCommand(&XpyApp); 
 *     // Process stream data command if received
 *     DBG002_XspyStreamData(&XpyApp);
 *   }
 * }
 * @endcode<BR> </p>  
 */
status_t DBG002_XspyStreamData (DBG002_XSpyDataType *XspyData);

/**
 * @brief  This API is used for Processing Execute Sync command. 
 *         User has to call this Api in application control loop to process execute sync command
 *         received from host.
 *
 * @param[in]   XspyDataPtr  This structure shall be part of the app instance handle. This structure
 *                        contains information data to be streamed. and buffer for storing the data
 *
 * @return      status_t <BR>
 * 			   DAVEApp_SUCCESS : If command processed successfully<BR>
 * 			   DBG002_NO_COMMAND_RECEIVED  : If Execute Sync command is not received<BR>
 * 				
 * 
 * <b>Reentrant: yes</b><BR>
 * <b>Sync/Async:  Synchronous</b>
 * <BR><P ALIGN="LEFT"><B>Example:</B>
 * @code 
 * #include <DAVE3.h>      //Declarations from DAVE3 Code Generation
 *
 * // A few globals.
 * uint8_t Speed = 0x01;
 * uint16_t Cur = 0x0102;
 * uint32_t Voltage = 0xff22ff33;
 * uint16_t Degree = 0xff34;
 * uint16_t Torque = 0xab34;
 * 
 * // Variable tables.
 * DBG002_VariableDescriptionType
 * Global1VarTable[5] = {
 *   XSPY_ADD_UINT8("Speed", Speed),
 * 	 XSPY_ADD_UINT16("Current", Cur),
 * 	 XSPY_ADD_UINT32("Voltage", Voltage),
 * 	 XSPY_ADD_UINT16("Degree", Degree),
 * 	 XSPY_ADD_UINT16("Torque", Torque)
 * };
 * 
 * DBG002_XSpyDataType XpyApp;
 * 
 * int main(void) {
 *   DAVE_Init();
 *   // Xspy init function
 *   DBG002_XspyInit(&XpyApp, Global1VarTable, 5, DBG002_GID_XSPY1);
 *    while(1) {
 * 	    // Process command if received
 *      DBG002_XspyProcessCommand(&XpyApp);
 *   }
 * }
 * @endcode<BR> </p>   
 */
status_t DBG002_XspyProcessCommand
(
  DBG002_XSpyDataType *XspyDataPtr
);

/**
 * @}
 */
 
#ifdef __cplusplus
}
#endif
 
#endif /* DBG002_XSPY_H_ */
