/*******************************************************************************
**																			  **
** Copyright (C) 2014 Infineon Technologies AG. All rights reserved.          **
**                                                                            **
** Infineon Technologies AG (Infineon) is supplying this software for use     **
** with Infineon's microcontrollers.									      **
** This file can be freely distributed within development tools that are      **
** supporting such microcontrollers.                                          **
**                                                                            **
** THIS SOFTWARE IS PROVIDED "AS IS".  NO WARRANTIES, WHETHER EXPRESS,        **
** IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES    **
** OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS      **
** SOFTWARE. INFINEON SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, **
** INCIDENTAL, OR CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.           **
**																			  **     
********************************************************************************
**                                                                            **
** PLATFORM : Infineon XMC4000/XMC1000 Series                                 **
**                                                                            **
** COMPILER : Compiler Independent                                            **
**                                                                            **
** AUTHOR   : App Developer                                                   **
**                                                                            **
** NOTE     : Any Changes made to this file will be overwritten during Code   **
**            Generation                                                      **
**                                                                            *
********************************************************************************
 **                           Revision History                                **  
 *******************************************************************************
 * 04 September 2012   v1.0.0    Initial version
 * 19 October   2012   v1.0.4    IO App instance number handling changed
 * 06 December  2012   v1.0.6    IO004 App is used for Enable pin
 * 03 April     2013   v1.0.8    SIN table size increased (256 ->1024)
 * 25 April     2013   v1.0.12   Added support for devices - XMC1302
 * 28 May       2013   v1.0.22   Added: Pseudo zero vector SVM and CORDIC option
 *                               for XMC13 device
 * 21 June      2013   v1.0.24   Added support for devices - XMC44 and XMC42
 * 24 July      2013   v1.0.26   Added support for phase current measurement,
 *                               Modified CORDIC Function,
 *                               Max amplitude limited based on SVM Mode.
 * 23 September 2013   v1.0.28   CCU8 slice constraint removed  
 * 26 November  2013   v1.0.30   Added Pseudo Zero Vector support for XMC4x 
 *                               device.
 * 30 January   2014   v1.0.36   Updated PWMSVM01.C and PWMSVM01.h file to 
 *                               static file.
 *                               Added Pseudo Zero Vector support for XMC4x device.
 *                               Updated Phase current measurement configuration 
 *                               and slice number for Phases.
 *                               XMC4000 Device support extended for XMC4500 AC step, 
 *                               XMC4400 AB Step and XMC4200 AB Step. 
 *                               DBG002 related MACRO has been updated.                         
 ******************************************************************************/
/**
 * @file PWMSVM01.h
 *
 * @brief This file contains the data structures and the function prototypes of
 *  the PWMSVM01 App. 
 */

#ifndef PWMSVM01_H_
#define PWMSVM01_H_

/* Support for C++ codebase */
#ifdef __cplusplus
extern "C" {
#endif
/*******************************************************************************
**                         Include Files                                      **
*******************************************************************************/
#include <DAVE3.h>

#include <uc_id.h>
#if (__TARGET_DEVICE__ == XMC45)
#include <XMC4500.h>
#elif (__TARGET_DEVICE__ == XMC44)
#include <XMC4400.h>
#elif ((__TARGET_DEVICE__ == XMC42) ||(__TARGET_DEVICE__ == XMC41) )
#include <XMC4200.h>
#elif (__TARGET_DEVICE__ == XMC11)
#include <XMC1100.h>
#elif (__TARGET_DEVICE__ == XMC12)
#include <XMC1200.h>
#elif (__TARGET_DEVICE__ == XMC13)
#include <XMC1300.h>
#else
#error "Unsupported XMC family"
#endif

/**
 * @ingroup PWMSVM01_constants
 * @{
 */
/*******************************************************************************
**                      Global Macro Definitions                              **
*******************************************************************************/
/**
 * This is the macro for sin scale to eliminate gain factor K
 */
#define PWMSVM01_KSINSCALE     (9949U)
/**
 * This is the macro for cos scale to eliminate gain factor K
 */
#define PWMSVM01_KCOSSCALE     (17231U)
/**
 * This is the macro for sixty degree count
 */
#define PWMSVM01_SIXTYDEG      (10922U)
/**
 * This is the macro for thirty degree count
 */
#define PWMSVM01_THIRTYDEG     (PWMSVM01_SIXTYDEG/2)
/**
 * This is the macro for angle scale
 */
#define PWMSVM01_ANGLESCALE    (8U)
/**
 * This is the macro for number of phase
 */
#define PWMSVM01_NUMBERPHASE   (3U)
/**
 * @}
 */
/*******************************************************************************
** GLOBAL VARIABLE DEFINITIONS: 
*******************************************************************************/
/**
 * @ingroup PWMSVM01_enumerations
 * @{
 */
/*******************************************************************************
 *                                ENUMERATIONS                                **
 ******************************************************************************/
/**
 * This enumerates the app state.
 */

typedef enum PWMSVM01_StateType
{
  /**
   * This is the default state after power on reset. App goes to UNINITIALIZED 
   * state after execution of the Deinit function.
   */
  PWMSVM01_UNINITIALIZED,
  /**
   * App is in INITIALIZED state after execution of the Init function
   */
  PWMSVM01_INITIALIZED,
  /**
   * App is in RUNNING state after execution of the Start function. 
   */
  PWMSVM01_RUNNING,
  /**
   * App is in STOPPED state after execution of the Stop function.
   */
  PWMSVM01_STOPPED
}PWMSVM01_StateType;

/**
 * This enumerates the PWM SVM type
 */
typedef enum PWMSVM01_PWMType
{
  /**
   * This is the pseudo null vector technique
   */
  PWMSVM01_PSEUDONULLVECTOR,
  /**
   * This is the standard SVM technique
   */
  PWMSVM01_STANDARD
  
}PWMSVM01_PWMType;

/**
 * This enumerates method for segment time calculation
 */
typedef enum PWMSVM01_SegmentTimeCalcMethod
{
  /**
   * This enumerates CORDIC use for segment time calculation 
   */
   PWMSVM01_USE_CORDIC,
   /**
    * This enumerates look up table use for segment time calculation
    */
   PWMSVM01_USE_LOOKUPTABLE

}PWMSVM01_SegmentTimeCalcMethod;

/**
 * This enumerates the error codes of the app which can be occured during 
 * run-time. 
 */
typedef enum PWMSVM01_ErrorType
{
  /**
   * This indicates that the function execution is not allowed in the current 
   * App State
   */
  /*
  GROUPID  #100
  MODULENAME = PWMSVM01
  */
  /*
  ERRCODESTRING = PWMSVM01_OPERATION_NOT_ALLOWED
  STRCODESTRING = Function execution is not allowed in the current App State
  */  
  PWMSVM01_OPERATION_NOT_ALLOWED = 1,
  /**
   * This indicates that input parameter is out of range. 
   */
  /*
  ERRCODESTRING = PWMSVM01_INVALID_PARAM
  STRCODESTRING = Input parameter is not valid
  */
  PWMSVM01_INVALID_PARAM,
  /**
   * This indicates that trap condition is detected.
   */
  /*
  ERRCODESTRING = PWMSVM01_TRAP_ERROR
  STRCODESTRING = Trap condition is detected
  */  
   PWMSVM01_TRAP_ERROR,
   /**
    * Function Entry
    */
   /*
   ERRCODESTRING = PWMSVM01_FUN_ENTRY
   STRCODESTRING = Entered function %s
   */
   PWMSVM01_FUN_ENTRY,
   /**
    *  Function Exit
    */
   /*
   ERRCODESTRING = PWMSVM01_FUN_EXIT
   STRCODESTRING = Exited function %s
   */
   PWMSVM01_FUN_EXIT   

 }PWMSVM01_ErrorType;
 
 /**
  * This enumerates the CC8 events
  */
typedef enum PWMSVM01_EventType
{
  /**
   * This is the period match interrupt. Interrupt will be set when timer  
   * reaches to the period register value
   */
  PWMSVM01_PERIODMATCH = 0,
  /**
   * This is the one match interrupt. Interrupt will be set when timer counts  
   * down to one.
   */
  PWMSVM01_ONEMATCH = 1,
  /**
   * This is the compare match interrupt while counting Up. Interrupt will be 
   * set when timer matches the compare register value while counting up.
   */
  PWMSVM01_UPCOMPMATCH = 2,
  /**
   * This is the compare match interrupt while counting down. Interrupt will be 
   * set when timer matches the compare register value while counting down.
   */
  PWMSVM01_DOWNCOMPMATCH = 3,
  /**
   * This is the Event 0 interrupt mapped to the External Start. Interrupt will 
   * be set when external start (event 0) edge is detected
   */
  PWMSVM01_EXTSTART = 8,
  /**
   * This is the Event 2 interrupt mapped to the External Trap. Interrupt will 
   * be set when external trap condition (event 2) is detected.
   */
  PWMSVM01_TRAP = 10
 }PWMSVM01_EventType;


/**
 * This enumerates the current measurement types
 */
typedef enum PWMSVM01_CMType
{
  /**
   * This enumerates hardware trigger for current measurement is not enabled.
   */
  PWMSVM01_NONE      = 0,
  /**
   * This enumerates the average current measurement
   */
  PWMSVM01_AVGCM      = 1,
  /**
   * This enumerates the dc link current measurement
   */
  PWMSVM01_DCLINKCM    = 2,
  /**
   * This enumerates the phase current measurement
   */
  PWMSVM01_PHASECM    = 3
 
}PWMSVM01_CMType;
  
/**
  * This enumerates the external trigger type forcurrent measurement
  */
typedef enum PWMSVM01_ExtTriggerType
{
  /**
   * This enumerates software trigger 
   */
  PWMSVM01_SWTRIGGER      = 0,
  /**
   * This enumerates hardware trigger 
   */
  PWMSVM01_HWTRIGGER      = 1
 
}PWMSVM01_ExtTriggerType;
  
/**
 * @}
 */
/**
 * @ingroup PWMSVM01_datastructures
 * @{
 */
 /******************************************************************************
  *                             STRUCTURES                                    **
  *****************************************************************************/
 /**
  * This structure holds the SVM parameters which change at run
  * time.
  */
typedef struct PWMSVM01_DynamicHandleType
{
  /**
   * Ta Segment Time for first half of PWM Period
   */  
  uint16_t Ta;
  /**
   * Tb Segment Time for first half of PWM Period
   */  
  uint16_t Tb;
  /**
   * Ta Segment Time for Second half of PWM Period
   */
  uint16_t Ta2;
  /**
   * Tb Segment Time for Second half of PWM Period
   */
  uint16_t Tb2;
  /**
   * Compare Value calculation  Variable
   */
  uint16_t V_TA;
  /**
   * Compare Value calculation  Variable
   */
  uint16_t V_TB;
  /**
   * Compare Value calculation  Variable
   */
  uint16_t V_TA2;
  /**
   * Compare Value calculation  Variable
   */
  uint16_t V_TB2;
  /**
   * Compare Value calculation  Variable
   */
  uint16_t V_TC;
  /**
   * Sector value 0-5
   */
  uint16_t Sector;
  /**
   * This variable give sub sector info for 3 active vector technique
   * 0-Less than Transition Angle
   * 1-Greater than Transition Angle
   */  
  uint16_t SubSector;
  /**
   * This is the state of the app.
   */
  PWMSVM01_StateType State;
  
}PWMSVM01_DynamicHandleType;


/**
 * This structure holds the GUI configurable parameters of this app.
 */
typedef struct PWMSVM01_HandleType
{
  /**
   * This is the CMSIS register structure pointer to the CCU8 kernel.
   */
  CCU8_GLOBAL_TypeDef*  GlobalPtr;
  /**
   * This is the CMSIS register structure pointer for the three CCU8 slices.
   */
  CCU8_CC8_TypeDef*  PhasePtr[3];
  /**
   * This is the CMSIS register structure pointer for CCU8 slice
   * which is used to trigger the ADC for current measurement.
   */
  CCU8_CC8_TypeDef*  CurrentTriggerPtr;
  /**
   * This is the CMSIS register structure pointer for CCU8 slice for 
   * Current measurement trigger.
   * This variable has been maintained for backward compatibility. 
   * Slice83Ptr & CurrentTriggerPtr hold the same address.  
   */
  CCU8_CC8_TypeDef* Slice83Ptr;
  /**
   * This is the pointer to the Dynamic Handle of the App.
   */
  PWMSVM01_DynamicHandleType*   DynamicHdlPtr;
  /**
   * This is the pointer to the IO002 App handle which is used to enable 
   * the inverter pin.
   */
  const IO004_HandleType* InverterPinPtr;
  /**
   * This is the dead time register value for all three phases.
   */
  uint16_t kDeadTime;
  /**
   * This is the Tmin value required for SVM algorithm to make sure that 
   * Ta or Tb is sufficient for the DC Link current measurement.
   */
  uint16_t kTmin;
  /**
   * This is the PWM frequency value.
   */
  uint32_t kPWMFrequency;
  /**
   * This is the period register value corresponding to the PWM frequency 
   * configured.
   */
  uint16_t kPeriod;
  /**
   * This is used to restrict maximum amplitude
   */
  uint16_t kMaxAmplitude;
  /**
   * This is used to configure synchronous start of the CCU8 slices.
   */
  uint16_t kSyncStart;
  /**
   * This is the prescaler value for the Dead Time Counter.
   */
  uint8_t kDeadTimePrescaler;
  /**
   * This is the passive level of the high and low side switches.
   */
  uint8_t   kPassiveLevel;
  /**
   * This is the CCU8 clock prescaler value.
   */
  uint8_t   kPrescaler;
  /**
   * This bit field saves the SVM type and SVM scheme. \n
   * 000: 7-segment Symmetric \n
   * 001: 7-Segment Asymmetric \n
   * 010: 5-segment Symmetric \n
   * 011: 5-Segment Asymmetric \n
   * 100: 3-Active pseudo zero vector SVM \n
   * 101: 4-Active pseudo zero vector SVM \n
   */
  unsigned kSVMConfig   :3;
  /**
   * This parameter tells PWM is asymmetric or not
   */
  uint8_t kAsymmetricPWM;
  /**
   * This parameter gives the segment time calculation type
   */
  uint8_t kSegTimeCalc;
  /**
   * This tells whether to enable period match interrupt \n
   * 0: disabled \n
   * 1: enabled
   */
  unsigned kEnablePerMatch :1;  
  /**
   * This tells whether to enable compare match interrupt
   * for ccu8 slice 3.Its enable when external trigger for
   * current measurement is selected.
   * 0: disabled \n
   * 1: enabled
   */
  unsigned kECurrentMeasInt  :1;
  /**
   * This tells whether to enable trap interrupt \n
   * 0: disabled \n
   * 1: enabled
   */
  unsigned kEnableTrapInterrupt :1;  
  /**
   * This tell whether to enable external trap feature \n
   * 0: disabled \n
   * 1: enabled
   */
  unsigned kEnableTrap :1;
  /**
   * This saves the inverter pin level. \n
   * 00: Inverter pin not required \n
   * 01: Active high pin \n
   * 10: Active low pin \n
   * 11: Reserved
   */
  unsigned kInverterPin : 2;
  /**
   * This configures the trap exit synchronization with the PWM signal is 
   * required or not.
  */
  uint8_t kTrapSync;
  /**
   * This parameter gives the selected SVM Mode
   * 0-Pseudo Zero Vector SVM
   * 1-Standard SVM
 */
  uint8_t kPWMType;
  /**
   * This indicates whether to exit from TRAP state automatically when TRAP
   * condition is removed or SW trigger is required to exit from the TRAP state.
  */
  uint8_t kTrapExitControl;
  /**
   * This parameter gives the active level of the Trap signal
  */
  uint8_t kTrapLevel;
  
  /**
   * This parameter gives the type of current measurement when external trigger 
   * is enabled.
   */
  uint8_t kExtTriggerCMType;
  
  /**
   * This parameter gives the type of trigger control for current measurement.
   */
  uint8_t kTriggerControl;
  /**
   * This parameter for enabling shadow transfer for consumed slices.
   */
  uint32_t kShadowTransfer;
  /**
   * This is the CCU8 slice number used for Phase W
   */
  uint32_t kPhWSliceNo;
  /**
   * This is the CCU8 slice number used for Phase V
   */
  uint32_t kPhVSliceNo;
  /**
   * This is the CCU8 slice number used for Phase U
   */
  uint32_t kPhUSliceNo;
  /**
   * This is the CCU8 slice number used for current measurement trigger
   */
  uint32_t kCurrentTrigSliceNo;
  
}PWMSVM01_HandleType;

/**
 * @}
 */
/**
 * @ingroup PWMSVM01_apidoc
 * @{
 */
/*******************************************************************************
          FUNCTION PROTOTYPES
*******************************************************************************/
/**
 * @brief This function will initialize CCU8x_CC8y global and slice registers
 * with user configured values.
 * @param[in] void 
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 * @code
   #include <DAVE3.h>
   int main (void)
   {
     DAVE_Init();   //PWMSVM01_Init is called within DAVE_Init()
     while(1);
     return 0;
   }
 * @endcode 
 */
void PWMSVM01_Init(void);

/**
 * @brief This function will reset CCU8x_CC8y global and slice registers and 
 * also the dynamic handle parameters.
 * @param[in] HandlePtr Handle of the PWMSVM01 App
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 * @code
   #include <DAVE3.h>
   int main (void)
   {
     DAVE_Init();   //PWMSVM01_Init is called within DAVE_Init()
     //Do something
     PWMSVM01_Deinit(&PWMSVM01_Handle0);
     while(1);
     return 0;
   }
 * @endcode 
 */
void PWMSVM01_Deinit(const PWMSVM01_HandleType* HandlePtr);

/**
 * @brief This function starts the App and initiates the synchronous start for 
 * the CC8 slices. It also sets the inverter enable pin.
 * @param[in] HandlePtr Handle of the PWMSVM01 App
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
   * @code
   #include <DAVE3.h>
   int main (void)
   {
     DAVE_Init();   //PWMSVM01_Init is called within DAVE_Init()
     PWMSVM01_Start(&PWMSVM01_Handle0); 
     //Do something
     while(1);
     return 0;
   }
 * @endcode 
 */
void PWMSVM01_Start(const PWMSVM01_HandleType* HandlePtr);

/**
 * @brief This function stops the App and stops the
 * PWM generation. It also resets the inverter enable pin.
 * @param[in] HandlePtr Handle of the PWMSVM01 App
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 * @code
   #include <DAVE3.h>
   int main (void)
   {
     DAVE_Init();   //PWMSVM01_Init is called within DAVE_Init()
     PWMSVM01_Start(&PWMSVM01_Handle0); 
     //Do something
     PWMSVM01_Stop(&PWMSVM01_Handle0); 
     while(1);
     return 0;
   }
 * @endcode
 */
void PWMSVM01_Stop(const PWMSVM01_HandleType* HandlePtr);

/**
 * @brief This is the SVM algorithm. It calls local functions to calculate and  
 * update the duty cycle of the three phases depending upon the
 * SVM type and SVM scheme. This function would be called from the interrupt 
 * handler.
 * @param[in] HandlePtr Handle of the PWMSVM01 App
 * @param[in] Amplitude Voltage to be applied to the motor.
 * 0 - 100% => 0- PWM Period Value
 * @param[in] Angle Angle of the rotor, 0 -360 degrees => 0 - 0xFFFF.
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 * @code
   #include <DAVE3.h>
   int main (void)
   {
     DAVE_Init();   //PWMSVM01_Init is called within DAVE_Init()
     PWMSVM01_Start(&PWMSVM01_Handle0); 
     while(1);
     return 0;
   }
   
   void PeriodMatchInterruptHandler(void)
   {
     PWMSVM01_SVM(&PWMSVM01_Handle0, 600, 30);
   }
 * @endcode
 */
void PWMSVM01_SVM
(
		const PWMSVM01_HandleType* HandlePtr,
		uint16_t Amplitude, 
		uint16_t Angle
		);

/**
 * @brief This function resets the trap flag. 
 * @param[in] HandlePtr Handle of the PWMSVM01 App
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 * @code
   #include <DAVE3.h>
   int main (void)
   {
     DAVE_Init();   //PWMSVM01_Init is called within DAVE_Init()
     PWMSVM01_Start(&PWMSVM01_Handle0); 
     while(1)
     {
        if(PWMSVM01_GetPendingEvent(&PWMSVM01_Handle0, 0, PWMSVM01_TRAP) == 1)
        {
          PWMSVM01_ResetTrapFlag(&PWMSVM01_Handle0);
        }
     }
     return 0;
   }
 * @endcode
 */
void PWMSVM01_ResetTrapFlag(const PWMSVM01_HandleType* HandlePtr);

/**
 * @brief This function enables the event for the specified phase. 
 * @param[in] HandlePtr Handle of the PWMSVM01 App
 * @param[in] PhaseNo This is the phase no 
 * 0:Phase U, \n 1: Phase V, \n 2: Phase W
 * @param[in] Event Interrupt number
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 * @code
   #include <DAVE3.h>
   int main (void)
   {
     DAVE_Init();   //PWMSVM01_Init is called within DAVE_Init()
     PWMSVM01_EnableEvent(&PWMSVM01_Handle0, 0, PWMSVM01_PERIODMATCH);
     PWMSVM01_Start(&PWMSVM01_Handle0); 
     while(1);
     return 0;
   }
 * @endcode
 */
void PWMSVM01_EnableEvent
(
    const PWMSVM01_HandleType* HandlePtr, 
    uint8_t PhaseNo, 
    PWMSVM01_EventType Event
);

/**
 * @brief This function clears the pending event for the specified phase 
 * @param[in] HandlePtr Handle of the PWMSVM01 App
 * @param[in] PhaseNo This is the phase no. 
 * 0:Phase U, \n 1: Phase V, \n 2: Phase W
 * @param[in] Event Interrupt number
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 * @code
   #include <DAVE3.h>
   int main (void)
   {
     DAVE_Init();   //PWMSVM01_Init is called within DAVE_Init()
     PWMSVM01_EnableEvent(&PWMSVM01_Handle0, 0, PWMSVM01_PERIODMATCH);
     PWMSVM01_Start(&PWMSVM01_Handle0); 
     while(1);
     return 0;
   }
   void PeriodMatchInterruptHandler(void)
   {
     PWMSVM01_ClearPendingEvent(&PWMSVM01_Handle0, 0, PWMSVM01_PERIODMATCH);
   }   
 * @endcode
 */
void PWMSVM01_ClearPendingEvent
(
    const PWMSVM01_HandleType* HandlePtr, 
    uint8_t PhaseNo, 
    PWMSVM01_EventType Event
);

/**
 * @brief This function checks if given event is set or not
 * @param[in] HandlePtr Handle of the PWMSVM01 App
 * @param[in] PhaseNo This is the phase no.
 * 0:Phase U, \n 1: Phase V, \n 2: Phase W
 * @param[in] Event Interrupt number
 * @return Status of the event
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 * @code
   #include <DAVE3.h>
   int main (void)
   {
     DAVE_Init();   //PWMSVM01_Init is called within DAVE_Init()
     PWMSVM01_EnableEvent(&PWMSVM01_Handle0, 0, PWMSVM01_PERIODMATCH);
     PWMSVM01_Start(&PWMSVM01_Handle0); 
     while(1);
     return 0;
   }
   void PeriodMatchInterruptHandler(void)
   {
     if(PWMSVM01_GetPendingEvent(&PWMSVM01_Handle0, 0, PWMSVM01_PERIODMATCH)==1)
     {
       //Do something
       PWMSVM01_ClearPendingEvent(&PWMSVM01_Handle0, 0, PWMSVM01_PERIODMATCH);
     }
   }   
 * @endcode
 */
uint8_t PWMSVM01_GetPendingEvent
(
    const PWMSVM01_HandleType* HandlePtr, 
    uint8_t PhaseNo, 
    PWMSVM01_EventType Event
);

/**
 * @brief This function disables the given event for the specified phase. 
 * @param[in] HandlePtr Handle of the PWMSVM01 App
 * @param[in] PhaseNo This is the phase no.
 * 0:Phase U, \n 1: Phase V, \n 2: Phase W
 * @param[in] Event Interrupt number
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 * @code
   #include <DAVE3.h>
   int main (void)
   {
     DAVE_Init();   //PWMSVM01_Init is called within DAVE_Init()
     PWMSVM01_DisableEvent(&PWMSVM01_Handle0, 0, PWMSVM01_PERIODMATCH);
     PWMSVM01_Start(&PWMSVM01_Handle0); 
     while(1);
     return 0;
   }
 * @endcode
 */
void PWMSVM01_DisableEvent
(
    const PWMSVM01_HandleType* HandlePtr, 
    uint8_t PhaseNo, 
    PWMSVM01_EventType Event
);

/**
 * @brief This function forcefully sets the given event by software. 
 * This function should not be used inside the interrupt handler. 
 * @param[in] HandlePtr Handle of the PWMSVM01 App
 * @param[in] PhaseNo This is the phase no.
 * 0:Phase U, \n 1: Phase V, \n 2: Phase W
 * @param[in] Event Interrupt number
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 * @code
   #include <DAVE3.h>
   int main (void)
   {
     DAVE_Init();   //PWMSVM01_Init is called within DAVE_Init()

     PWMSVM01_Start(&PWMSVM01_Handle0); 
     PWMSVM01_SetPendingEvent(&PWMSVM01_Handle0, 0, PWMSVM01_PERIODMATCH);
     while(1);
     return 0;
   }
 * @endcode
 */
void PWMSVM01_SetPendingEvent
(
    const PWMSVM01_HandleType* HandlePtr, 
    uint8_t PhaseNo, 
    PWMSVM01_EventType Event
);

/**
 * @}
 */

#include "PWMSVM01_Conf.h"
#include "PWMSVM01_Extern.h"

/* Support for C++ codebase */
#ifdef __cplusplus
}
#endif 

#endif /* PWMSVM01_H_ */


