/**
 * @file PMSMFOCSL01.c
 *
 * @brief This file contains the implementation of all public and private 
 * functions of the PMSMFOCSL01 App.
 * This App implements the Sensor less Field Oriented Control of 3- Phase PMSM 
 * Motor. 
 */

/*******************************************************************************
** INCLUDE FILES:
*******************************************************************************/
#include <DAVE3.h>
/*******************************************************************************
** PRIVATE MACRO DEFINITIONS:
*******************************************************************************/
/**
  * @ingroup PMSMFOCSL01_constants
  * @{
  */
/*These definitions are included here to avoid compilation errors,
 since the DBG002 app is not part of the project. All the macros are defined
 as empty*/
#ifndef _DBG002_H_

#define DBG002_RegisterCallBack(A,B,C)
#define DBG002_I(e)
#define DBG002_IG(e,g)
#define DBG002_IH(e,h)
#define DBG002_IP(e,p)
#define DBG002_IGH(e,g,h)
#define DBG002_IGP(e,g,p)
#define DBG002_IHP(e,h,p)
#define DBG002_IGHP(e,g,h,p)
#define DBG002_N(e)
#define DBG002_NG(e,g)
#define DBG002_NH(e,h)
#define DBG002_NP(e,p)
#define DBG002_NGH(e,g,h)
#define DBG002_NGP(e,g,p)
#define DBG002_NHP(e,h,p)
#define DBG002_NGHP(e,g,h,p)
#define DBG002_ID(e)
#define DBG002_IS(e)
#define DBG002_ISG(e,g)
#define DBG002_SAFETY_CRITICAL(groupid,messageid,length,value)
#define DBG002_CRITICAL(groupid,messageid,length,value)
#define DBG002_ERROR(groupid,messageid,length,value)
#define DBG002_WARNING(groupid,messageid,length,value)
#define DBG002_INFO(groupid,messageid,length,value)
#define DBG002_TRACE(groupid,messageid,length,value)
#define DBG002_FUNCTION_ENTRY(GID, Status)
#define DBG002_FUNCTION_EXIT(GID, Status)

#define DBG002_MESSAGEID_LITERAL

#endif/* End of defintions of dummy Debug Log macros*/

#define APP_GID DBG002_GID_PMSMFOCSL01

#if (UC_SERIES == XMC13)
#define ADC_AI0040    ( *(unsigned int*) 0x480340E0 )
#define ADC_AI0041    ( *(unsigned int*) 0x480340E4 )
#endif

/**
 * @}
 */


/**
* @cond INTERNAL_DOCS
*/
/*******************************************************************************
** PRIVATE TYPE DEFINITIONS:
*******************************************************************************/

/*******************************************************************************
** PRIVATE FUNCTION DECLARATIONS:
*******************************************************************************/

/**
 * @brief This function enables the IRQ  and sets the prioriy ans sub priority.\n
 * @param[in] HandlePtr Pointer to  PMSMFOCSL01_ISRHandleType  structure
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 */
void PMSMFOCSL01_lISRInit(const PMSMFOCSL01_ISRHandleType* HandlePtr);
/**
 * @brief This function initializes all the dynamic Motor parameters.\n
 * @param[in] HandlePtr Handle of the PMSMFOCSL01 App
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 */
void PMSMFOCSL01_lMotoParamInit (const PMSMFOCSL01_HandleType* HandlePtr );
/**
 * @brief This function will initialize the motor parameters,App state and
 * initialize the queue entry.\n
 * @param[in] HandlePtr Handle of the PMSMFOCSL01 App
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 */
void PMSMFOCSL01_lInit(const PMSMFOCSL01_HandleType* HandlePtr);

/**
 * @brief This interrupt handler is shared by Trap and One Match.  
 * Motor is stopped when trap interrupt is detected.
 * One match interrupt is used to handle the CCR rate of the PWM period match 
 * interrupt.
 * @param[in] HandlePtr Handle of the PMSMFOCSL01 App
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 */
void PMSMFOCSL01_lTrapVOneMatchInterruptHandler
(const PMSMFOCSL01_HandleType* HandlePtr);

/**
 * @brief This function will be called when
 * wrong hall event is generated. This will in turn stop the motor.\n
 * @param[in] HandlePtr Pointer to  PMSMFOCSL01_HandleType  structure
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 */
void PMSMFOCSL01_PeripheralStart(const PMSMFOCSL01_HandleType* HandlePtr);

/**
 * @brief This function will be called when every CCU8 Compare Match interrupt 
 * is detected for ADC measurement trigger\n
 * @param[in] HandlePtr Handle of the PMSMFOCSL01 App
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 */
void PMSMFOCSL01_lCurrentMeasureInterruptHandler
(const PMSMFOCSL01_HandleType* HandlePtr); 

/**
 * @brief This is user define interrupt handler for period match interrupt\n
 * @param[in] void
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 */
void PMSMFOCSL01_PWMPeriodMatchISRA(void);
/**
 * @brief This is user define interrupt handler for trap and phase V one match 
 * interrupts\n
 * @param[in] void
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 */
void PMSMFOCSL01_TrapPhaseVOneMatchISRA(void);
/**
 * @brief This is user define interrupt handler for current measurement \n
 * @param[in] void
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 */
void PMSMFOCSL01_CurrentMeasureISRA(void);

#if (PMSMFOCSL01_NUM_INSTANCES == 2U) 
/**
 * @brief This is user define interrupt handler for period match interrupt\n
 * @param[in] void
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 */
void PMSMFOCSL01_PWMPeriodMatchISRB(void);
/**
 * @brief This is user define interrupt handler for trap and phase V one match 
 * interrupts\n
 * @param[in] void
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 */
void PMSMFOCSL01_TrapPhaseVOneMatchISRB(void);
/**
 * @brief This is user define interrupt handler for current measurement \n
 * @param[in] void
 * @return void
 * <BR>
 *
 * <b>Reentrancy:  Yes</b><BR>
 * <b>Sync/Async:  Synchronous</b><BR>
 */
void PMSMFOCSL01_CurrentMeasureISRB(void);

#endif

/*******************************************************************************
** GLOBAL CONSTANT DEFINITIONS:
*******************************************************************************/

/*******************************************************************************
** GLOBAL VARIABLE DEFINITIONS:
*******************************************************************************/

/*******************************************************************************
** PRIVATE CONSTANT DEFINITIONS:
*******************************************************************************/

/*******************************************************************************
** FUNCTION LIKE MACRO DEFINITIONS:
*******************************************************************************/

/*******************************************************************************
** PRIVATE VARIABLES:
*******************************************************************************/

/*******************************************************************************
** PRIVATE FUNCTION DEFINITIONS:
*******************************************************************************/

/*
 *This function will initialize the motor parameters,App state
 */ 
void PMSMFOCSL01_lInit(const PMSMFOCSL01_HandleType* HandlePtr)
{
  status_t Status;

  HandlePtr->H_Ptr->StartSpeedRef  = 
  (int32_t)HandlePtr->kUserStartSpeedRef * HandlePtr->kMotDirection;
  HandlePtr->H_Ptr->EndSpeedRef    = 
  (int32_t)HandlePtr->kUserEndSpeedRef * HandlePtr->kMotDirection;
  HandlePtr->H_Ptr->SpeedThreshold = (int32_t)HandlePtr->kUserStartSpeedThreshold;                                                    
  HandlePtr->H_Ptr->MotorDirection = HandlePtr->kMotDirection;
  HandlePtr->H_Ptr->RampCounter    = HandlePtr->kRampCounter;
  /*Initialise parameters of the motor*/
  PMSMFOCSL01_lMotoParamInit (HandlePtr);
   
  /*if CCR is greater than 1, enabled  One Match or Period match interrupt based
   on SVM config*/
  if((HandlePtr->kCurrentControlRate > 1U) ||
     (PMSMFOCSL01_TRUE == HandlePtr->kBootStrapEnable)){
    if((HandlePtr->SVMAppPtr->kSVMConfig == 4U) ||
      (HandlePtr->SVMAppPtr->kSVMConfig == 5U)){
      SET_BIT(HandlePtr->SVMAppPtr->PhasePtr[1]->INTE, CCU8_CC8_INTE_PME_Pos);
    }else{
      SET_BIT(HandlePtr->SVMAppPtr->PhasePtr[1]->INTE, CCU8_CC8_INTE_OME_Pos);
    }
  }  
  
  /*Make queue entries for current measurement.*/ 
  ADC002_InitializeQueue(HandlePtr->ADCReqSrcPtrA);
  /*Flush the queue entry before voltage compensation entry.*/ 
  ADC002_FlushQueue(HandlePtr->ADCReqSrcPtrB);
  /*Make queue entries for voltage compensation.*/ 
  ADC002_InitializeQueue(HandlePtr->ADCReqSrcPtrB);
  /*Flush the queue entry before pot measurement entry.*/ 
  ADC002_FlushQueue(HandlePtr->ADCReqSrcPtrC);
  /*Make queue entries for pot measurement.*/       
  Status = ADC002_InitializeQueue(HandlePtr->ADCReqSrcPtrC); 

  /*update the Motor status to OFF*/
  HandlePtr->H_Ptr->MotorStatus = 1U;

  if(Status == (uint32_t)DAVEApp_SUCCESS)
  {
    /*Change state to initialise */
    HandlePtr->H_Ptr->State = PMSMFOCSL01_INITIALIZED;
  }
}
/*
 * This function Initialise all the dynamic Motor parameters.
 */
void PMSMFOCSL01_lMotoParamInit (const PMSMFOCSL01_HandleType* HandlePtr )
{
  uint32_t Tempbuf;
  
  HandlePtr->H_Ptr->SpeedControlRate     = 0U;
  HandlePtr->H_Ptr->CurrentControlRate   = 0U;

  HandlePtr->H_Ptr->Amplitude            = 0U;
  HandlePtr->H_Ptr->Angle                = (int16_t)0;




  HandlePtr->H_Ptr->RampScheduler        = (uint16_t)0; 
  HandlePtr->H_Ptr->MotorStatus          = (uint16_t)1; 

  if ((HandlePtr->H_Ptr->StartSpeedRef) > (HandlePtr->H_Ptr->EndSpeedRef))
  {
      if (HandlePtr->H_Ptr->EndSpeedRef > 0)
      {
          HandlePtr->H_Ptr->SpeedRef = (int32_t)HandlePtr->H_Ptr->EndSpeedRef ;
      }
      else
      {
          HandlePtr->H_Ptr->SpeedRef = (int32_t)HandlePtr->H_Ptr->StartSpeedRef ;
      }
  }
  else
  {
    if(HandlePtr->H_Ptr->EndSpeedRef > 0)
    {
      HandlePtr->H_Ptr->SpeedRef = (int32_t)HandlePtr->H_Ptr->StartSpeedRef ;
    }
    else
    {
      HandlePtr->H_Ptr->SpeedRef = (int32_t)HandlePtr->H_Ptr->EndSpeedRef ;
    }
  }
  Tempbuf = ((HandlePtr->H_Ptr->IqStartup <<16)& PMSMFOCSL01_INTEGRALBUF_MASK);
  if(HandlePtr->H_Ptr->EndSpeedRef > 0)
  {
    HandlePtr->IqPIPtr->Ibuf = (int32_t)Tempbuf;    
  }
  else
  {
    HandlePtr->IqPIPtr->Ibuf = (int32_t)-1 * (int32_t)Tempbuf;    
  }

  HandlePtr->H_Ptr->Ia             = 0;
  HandlePtr->H_Ptr->Ib             = 0;
   
  HandlePtr->H_Ptr->Ialpha         = 0;
  HandlePtr->H_Ptr->Ibeta          = 0;
    
  HandlePtr->H_Ptr->Id             = 0;
  HandlePtr->H_Ptr->Iq             = 0;
  
  HandlePtr->H_Ptr->Valpha         = 0;
  HandlePtr->H_Ptr->Vbeta          = 0;
  
  HandlePtr->H_Ptr->Vd             = 0;
  HandlePtr->H_Ptr->Vq             = 0;
  
  HandlePtr->H_Ptr->Angle          = 0;
  HandlePtr->H_Ptr->Amplitude      = 0U;
  
  HandlePtr->StartupHPtr->Angle    = 0;
  
  HandlePtr->SpeedPT1Ptr->PT1buf   = 0;
  HandlePtr->ValphaPT12Ptr->PT1buf = 0;
  HandlePtr->VbetaPT12Ptr->PT1buf  = 0;
  
  HandlePtr->SpeedPIPtr->Ibuf      = 0;
  HandlePtr->IdPIPtr->Ibuf         = 0;
  HandlePtr->H_Ptr->MotorSpeed     = 0;

  //30.10.2016
  HandlePtr->H_Ptr->Xc		= 0;
  HandlePtr->H_Ptr->Ys		= 0;
  //HandlePtr->H_Ptr->EAlpha = HandlePtr->H_Ptr->FluxAlpha*HandlePtr->H_Ptr->Ys;
  //HandlePtr->H_Ptr->EBeta = HandlePtr->H_Ptr->FluxBeta*HandlePtr->H_Ptr->Xc;
  HandlePtr->H_Ptr->MotorSpeedR	= 0;
  HandlePtr->H_Ptr->FluxAngle1	= 0;
} 

/*******************************************************************************
**                      ISRs AND HANDLERS                                     **
*******************************************************************************/

/*
 * This function configure the the NVIC Node.
 */
void PMSMFOCSL01_lISRInit(const PMSMFOCSL01_ISRHandleType* HandlePtr) 
{
  /*set the priority and sub priority*/
#if (UC_FAMILY==XMC4)
  NVIC_SetPriority((IRQn_Type)HandlePtr->Node_Id, 
                   NVIC_EncodePriority(NVIC_GetPriorityGrouping(),
           (uint32_t)HandlePtr->Priority,(uint32_t)HandlePtr->Sub_Pri));
#elif (UC_FAMILY==XMC1)
  NVIC_SetPriority( (IRQn_Type)HandlePtr->Node_Id,(uint32_t)HandlePtr->Priority);
#endif 
  /*Enable the IRQ*/
  NVIC_EnableIRQ((IRQn_Type)HandlePtr->Node_Id);      
} 

/*
 * This is Trap and Phase V one match Interrupt Handler
 */
void PMSMFOCSL01_lTrapVOneMatchInterruptHandler
(const PMSMFOCSL01_HandleType* HandlePtr)
{
  /*Check the trap interrupt status flag(Event 2) */
  if((HandlePtr->SVMAppPtr->PhasePtr[0]->INTS & CCU8_CC8_INTS_E2AS_Msk) != 0U)
  {
    /*This function stops motor as trap occurs*/
    PMSMFOCSL01_MotorStop(HandlePtr);
    /* Set Motor Status Trap Flag*/
    SET_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_TRAP_Pos);
    PWMSVM01_ClearPendingEvent(HandlePtr->SVMAppPtr, 
                               (uint8_t)HandlePtr->SVMAppPtr->kPhUSliceNo,
                               PWMSVM01_TRAP);
  }
  /*Check PWM one match interrupt of slice 1 status flag*/
  if((HandlePtr->SVMAppPtr->PhasePtr[1]->INTS & (uint32_t)HandlePtr->kCCRIntSelect) != 0U)
  {
    if(PMSMFOCSL01_BOOTSTRAP == HandlePtr->H_Ptr->State) 
    {
      /*Software reset of periodmatch interrupt*/
      HandlePtr->SVMAppPtr->PhasePtr[1]->SWR |= (uint32_t)HandlePtr->kCCRIntSelect;

      if(HandlePtr->H_Ptr->BootCount++ > HandlePtr->kBootStrapCount)
      {
        if(HandlePtr->kCurrentControlRate <= 1U)
    	{
    	  if((PMSMFOCSL01_PSEUDO3V == HandlePtr->SVMAppPtr->kSVMConfig) || 
    	     (PMSMFOCSL01_PSEDO4V == HandlePtr->SVMAppPtr->kSVMConfig))
    	  {
    	    CLR_BIT(HandlePtr->SVMAppPtr->PhasePtr[1]->INTE,
    	    		CCU8_CC8_INTE_PME_Pos);
    	  }
    	  else
    	  {
    	    CLR_BIT(HandlePtr->SVMAppPtr->PhasePtr[1]->INTE,
    	    		CCU8_CC8_INTE_OME_Pos);
    	  }
        }

    	/*Stop and Reset CCU8 Timers*/
        HandlePtr->SVMAppPtr->PhasePtr[0]->TCCLR = (CCU8_CC8_TCCLR_TRBC_Msk|
                                                        CCU8_CC8_TCCLR_TCC_Msk);
        HandlePtr->SVMAppPtr->PhasePtr[1]->TCCLR = (CCU8_CC8_TCCLR_TRBC_Msk|
                                                        CCU8_CC8_TCCLR_TCC_Msk);
        HandlePtr->SVMAppPtr->PhasePtr[2]->TCCLR = (CCU8_CC8_TCCLR_TRBC_Msk|
                                                        CCU8_CC8_TCCLR_TCC_Msk);
        HandlePtr->SVMAppPtr->CurrentTriggerPtr->TCCLR = (CCU8_CC8_TCCLR_TRBC_Msk|
                                                         CCU8_CC8_TCCLR_TCC_Msk);                                                          

    	/*Reset CCU8 bit in CCUCON*/
        SCU_GENERAL->CCUCON &= ~((uint32_t)(HandlePtr->SVMAppPtr->kSyncStart));

        /*Enable period match of PhaseU*/
        SET_BIT(HandlePtr->SVMAppPtr->PhasePtr[0]->INTE,
        		  CCU8_CC8_INTE_PME_Pos);

    	/*Call Peripheral Start*/
        PMSMFOCSL01_PeripheralStart(HandlePtr);

        HandlePtr->H_Ptr->BootCount = 0U;

        CLR_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_BS_Pos);
      }
    } else {  
      HandlePtr->H_Ptr->CurrentControlRate++;
      if(HandlePtr->H_Ptr->CurrentControlRate >= HandlePtr->kCurrentControlRate)
      {
        HandlePtr->SVMAppPtr->PhasePtr[0]->INTE |= 0x1U;
        HandlePtr->SVMAppPtr->PhasePtr[0]->SWR  |= 0x1U;
        HandlePtr->H_Ptr->CurrentControlRate     = 0U;
      }
      HandlePtr->SVMAppPtr->PhasePtr[1]->SWR |= (uint32_t)HandlePtr->kCCRIntSelect;
    }
  } 
}

#if (UC_FAMILY==XMC4)
/*
 * This is Current Measurment Timer Period Match  Interrupt Handler
 */
void PMSMFOCSL01_lCurrentMeasureInterruptHandler
(const PMSMFOCSL01_HandleType* HandlePtr)
{
  PWMSVM01_ClearPendingEvent (HandlePtr->SVMAppPtr,
                              (uint8_t)HandlePtr->SVMAppPtr->kCurrentTrigSliceNo, 
                              PWMSVM01_UPCOMPMATCH);
}
#endif

/*
 *This is user define interrupt handler for period match interrupt 
 */
void PMSMFOCSL01_PWMPeriodMatchISRA(void)
{
  
  
  PMSMFOCSL01_ReadCurrent(&PMSMFOCSL01_HandleA);
  /*if CCR is greater than 1, disable  Period Match interrupt*/
  if (PMSMFOCSL01_HandleA.kCurrentControlRate > 1U) {
      CLR_BIT(PMSMFOCSL01_HandleA.SVMAppPtr->PhasePtr[0]->INTE,
      CCU8_CC8_INTE_PME_Pos);
  }
  if (PMSMFOCSL01_TRUE == PMSMFOCSL01_HandleA.Enable.Pot){
      /* Measure the pot value and update the end speed reference value */
      PMSMFOCSL01_PotMeasurement(&PMSMFOCSL01_HandleA);
  }
  /*update speed reference value */
  PMSMFOCSL01_SpeedRampup(&PMSMFOCSL01_HandleA);

  /*FOC Calculation */
  PMSMFOCSL01_FOCCalculation(&PMSMFOCSL01_HandleA);

  if (PMSMFOCSL01_TRUE == PMSMFOCSL01_HandleA.Enable.VoltageComp) {
      /* Measure the dc link value and update the voltage amplitude */
      PMSMFOCSL01_VoltageCompensation(&PMSMFOCSL01_HandleA);
  }

  /* This value is used for Current Reconstruction*/
  PMSMFOCSL01_HandleA.H_Ptr->SectorStore =
  PMSMFOCSL01_HandleA.SVMAppPtr->DynamicHdlPtr->Sector;

  /*call SVM*/
  PWMSVM01_SVM(PMSMFOCSL01_HandleA.SVMAppPtr,
               (uint16_t)PMSMFOCSL01_HandleA.H_Ptr->Amplitude,
               (uint16_t)PMSMFOCSL01_HandleA.H_Ptr->Angle);
  
   
}

/*
 *This is user define interrupt handler for trap 
 */
void PMSMFOCSL01_TrapPhaseVOneMatchISRA(void)
{   
  
  /*
   * This is trap and Phase V one match interrupt handler
   */
  PMSMFOCSL01_lTrapVOneMatchInterruptHandler(&PMSMFOCSL01_HandleA);
  
   
}
/*
 *This is user define interrupt handler for current measurement
 */
#if (UC_FAMILY==XMC4)
void PMSMFOCSL01_CurrentMeasureISRA(void)
{
  
  /*
   * This is current measurement interrupt handler
   */
  PMSMFOCSL01_lCurrentMeasureInterruptHandler(&PMSMFOCSL01_HandleA);
   
}
#endif

#if (PMSMFOCSL01_NUM_INSTANCES == 2U)
/*
 *This is user define interrupt handler for period match interrupt 
 */
void PMSMFOCSL01_PWMPeriodMatchISRB(void)
{
  
  
  PMSMFOCSL01_ReadCurrent(&PMSMFOCSL01_HandleB);
  /*if CCR is greater than 1, disable  Period Match interrupt*/
  if(PMSMFOCSL01_HandleB.kCurrentControlRate > 1) {
  CLR_BIT(PMSMFOCSL01_HandleB.SVMAppPtr->PhasePtr[0]->INTE, 
          CCU8_CC8_INTE_PME_Pos);
  }
  if(PMSMFOCSL01_TRUE == PMSMFOCSL01_HandleB.Enable.Pot){
    /* Measure the pot value and update the end speed reference value */
  PMSMFOCSL01_PotMeasurement(&PMSMFOCSL01_HandleB);
  }
  /*update speed reference value */
  PMSMFOCSL01_SpeedRampup(&PMSMFOCSL01_HandleB);

  /*FOC Calculation */
  PMSMFOCSL01_FOCCalculation(&PMSMFOCSL01_HandleB);

  if(PMSMFOCSL01_TRUE == PMSMFOCSL01_HandleB.Enable.VoltageComp) {
    /* Measure the dc link value and update the voltage amplitude */
    PMSMFOCSL01_VoltageCompensation(&PMSMFOCSL01_HandleB);
  }

  /* This value is used for Current Reconstruction*/
  PMSMFOCSL01_HandleB.H_Ptr->SectorStore =
  PMSMFOCSL01_HandleB.SVMAppPtr->DynamicHdlPtr->Sector;

  /*call SVM*/
  PWMSVM01_SVM(PMSMFOCSL01_HandleB.SVMAppPtr,
               (uint16_t)PMSMFOCSL01_HandleB.H_Ptr->Amplitude,
               (uint16_t)PMSMFOCSL01_HandleB.H_Ptr->Angle);
  
   
}

/*
 *This is user define interrupt handler for trap 
 */
void PMSMFOCSL01_TrapPhaseVOneMatchISRB(void)
{   
  
  /*
   * This is trap and Phase V one match interrupt handler
   */
  PMSMFOCSL01_lTrapVOneMatchInterruptHandler(&PMSMFOCSL01_HandleB);
  
   
}
/*
 *This is user define interrupt handler for current measurement
 */
#if (UC_FAMILY==XMC4)
void PMSMFOCSL01_CurrentMeasureISRB(void)
{
  
  /*
   * This is current measurement interrupt handler
   */
  PMSMFOCSL01_lCurrentMeasureInterruptHandler(&PMSMFOCSL01_HandleB);
   
}
#endif
#endif

/*******************************************************************************
** PUBLIC FUNCTION DEFINITIONS:
*******************************************************************************/
/*
 * This will initialize motor parameters and interrupt nodes.
 */
void PMSMFOCSL01_Init(void)
{
#if (UC_SERIES == XMC13)
  static volatile uint32_t count=0U;
#endif
  uint32_t Count;    /* Loop Counter */ 

  
  /*Initialization of 'PWMSVM01' App.*/
  PWMSVM01_Init();
  
  /*Initialization of 'ADC002' App.*/        
  ADC002_Init();
  
  for (Count = 0U; Count < PMSMFOCSL01_NUM_INSTANCES; Count++)
  {  

    /*Initialize the motor parameters for instance */ 
    PMSMFOCSL01_lInit(PMSMFOCSL01_HandleArray[Count]);
  
    /*Initialize the PWM ISR for instance */ 
    PMSMFOCSL01_lISRInit(PMSMFOCSL01_PMISRHandleArray[Count]);  

    /*Initialize the Trap ISR for instance */ 
    if (((PMSMFOCSL01_TRUE == (PMSMFOCSL01_HandleArray[Count]->Enable.Trap)) &&
       ((PMSMFOCSL01_ISRHandleType*)NULL != PMSMFOCSL01_TrapISRHandleArray[Count]))
       || (PMSMFOCSL01_TRUE == (PMSMFOCSL01_HandleArray[Count]->kBootStrapEnable))
       || (PMSMFOCSL01_HandleArray[Count]->kCurrentControlRate > 1U))
    {
	   PMSMFOCSL01_lISRInit(PMSMFOCSL01_TrapISRHandleArray[Count]);
    }  
    #if (UC_SERIES == XMC13) 
    /*ADC Workaround AI004*/
    VADC -> GLOBCFG = 0x80038000;
    for(count = 0U;count < 1000U;count++);
    while((SHS0->SHSCFG & SHS_SHSCFG_STATE_Msk) == (0x0011U <<SHS_SHSCFG_STATE_Pos));
    ADC_AI0040 = 0x80008000;
    ADC_AI0041 = 0x80008000;
    #endif
  }  
  
   
}  

void PMSMFOCSL01_PeripheralStart(const PMSMFOCSL01_HandleType* HandlePtr)
{
    /*Start the PWM generation*/
    PWMSVM01_Start(HandlePtr->SVMAppPtr);
    /*Change the App state */
    if(PMSMFOCSL01_TRUE == HandlePtr->Enable.VFStartup)
    {
      HandlePtr->H_Ptr->State = PMSMFOCSL01_OPENLOOP;
      SET_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_OL_Pos);
    }
    else
    {
      HandlePtr->H_Ptr->State = PMSMFOCSL01_RUNNING;
      SET_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_FOC_Pos);
    }
}
/*
 * This will start the motor.
 */
status_t PMSMFOCSL01_MotorStart(const PMSMFOCSL01_HandleType* HandlePtr)
{
  uint16_t Offset[16];
  uint16_t AmpOffset = 0U;
  uint8_t  Count; 
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  

  if ((PMSMFOCSL01_INITIALIZED ==HandlePtr->H_Ptr->State) ||
      (PMSMFOCSL01_STOPPED == HandlePtr->H_Ptr->State))
  {
    /*Initialise all the dynamic Motor parameters*/
    PMSMFOCSL01_lMotoParamInit(HandlePtr);
    /* Clear Motor Status  flags*/
    HandlePtr->H_Ptr->MotorStatus = 0U;
    /* Set MotorStatus on flag*/
    SET_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_ON_Pos);
    if(PMSMFOCSL01_TRUE == HandlePtr->kAmpBiasVoltCal){
    /* ADC bias voltage measurement */
      for(Count = 0U; Count < PMSMFOCSL01_ADCCAL_COUNT; Count++)
      {
        ADC002_TriggerEvent(HandlePtr->ADCReqSrcPtrA);
        Status = ADCCH001_GetResult(HandlePtr->ADCReqSrcPtrA->kChAHandlerPtr,
                                    &Offset[Count]);
        AmpOffset += Offset[Count];
      }
      HandlePtr->H_Ptr->AmpOffset = (uint32_t)AmpOffset/PMSMFOCSL01_ADCCAL_COUNT;
    } else {
      HandlePtr->H_Ptr->AmpOffset = HandlePtr->kAmpOffset;
    }
    
    if(PMSMFOCSL01_TRUE == HandlePtr->kBootStrapEnable)
    {
      /*Disable period match of PhaseU*/
      CLR_BIT(HandlePtr->SVMAppPtr->PhasePtr[0]->INTE, CCU8_CC8_INTE_PME_Pos);

 	  /*BootStrapping*/
      HandlePtr->SVMAppPtr->PhasePtr[0]->CR1S  = 0xFFFFU;
      HandlePtr->SVMAppPtr->PhasePtr[1]->CR1S  = 0xFFFFU;
      HandlePtr->SVMAppPtr->PhasePtr[2]->CR1S  = 0xFFFFU;
      HandlePtr->SVMAppPtr->GlobalPtr->GCSS   |= (CCU4_GCSS_S0SE_Msk|
 			                                      CCU4_GCSS_S1SE_Msk|
 			                                      CCU4_GCSS_S2SE_Msk);

      /*Clear Idle bit of CCU8 Slices*/
      HandlePtr->SVMAppPtr->GlobalPtr->GIDLC  |= (CCU8_GIDLC_CS0I_Msk|
 			                                      CCU8_GIDLC_CS1I_Msk|
 			                                      CCU8_GIDLC_CS2I_Msk|
 			                                      CCU8_GIDLC_CS3I_Msk);

      /*Synchronous start of all the three phases */
      SCU_GENERAL->CCUCON |= (uint32_t)(HandlePtr->SVMAppPtr->kSyncStart);

      SET_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_BS_Pos);

      HandlePtr->H_Ptr->State = PMSMFOCSL01_BOOTSTRAP;
    }
    else
    {
      PMSMFOCSL01_PeripheralStart(HandlePtr);
    }
    Status = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This will stop the motor.
 */
status_t PMSMFOCSL01_MotorStop(const PMSMFOCSL01_HandleType* HandlePtr)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  

  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    /*Stop the PWM signal generation*/
    PWMSVM01_Stop (HandlePtr->SVMAppPtr);
  
    PMSMFOCSL01_lMotoParamInit(HandlePtr);
  
    /*Clear MotorStatus ON flag and RAMP flag*/
    CLR_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_ON_Pos);
    CLR_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_RAMP_Pos);
    CLR_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_FOC_Pos);
    CLR_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_OL_Pos);
    /*Set MotorStatus off flag*/
    SET_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_OFF_Pos);
    /*Reset speed reference value*/
    HandlePtr->H_Ptr->SpeedRef = (int32_t)0;
    /*Change the App sate to Stopped*/
    HandlePtr->H_Ptr->State    = PMSMFOCSL01_STOPPED;
    Status                     = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This will set the motor end speed reference.
 */
status_t PMSMFOCSL01_SetMotorEndSpeed(const PMSMFOCSL01_HandleType* HandlePtr,
                                            int32_t                 SetSpeed)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  int32_t  TempSpeed;
  
   if (HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
   {
	 if((SetSpeed <= (int32_t)PMSMFOCSL01_POWER215) && (SetSpeed >= (-1*(int32_t)PMSMFOCSL01_POWER215)))
 	 {
 	   TempSpeed=((SetSpeed*(int32_t)PMSMFOCSL01_POWER215)/HandlePtr->kNormSpeed);
 	   HandlePtr->H_Ptr->EndSpeedRef = TempSpeed;
 	   Status = (uint32_t)DAVEApp_SUCCESS;
 	 }
 	 else
 	 {
 	   Status = (uint32_t)PMSMFOCSL01_INVALID_PARAM;
 	   DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
 	 }
   }
   else
   {
     DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
   }
   
  return Status;
}
/*
 * This will set the motor speed reference.
 */
status_t PMSMFOCSL01_SetMotorStartSpeed(const PMSMFOCSL01_HandleType* HandlePtr,
                                              int32_t                 SetSpeed)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  int32_t  TempSpeed;

  
  if (HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
	 if((SetSpeed <= (int32_t)PMSMFOCSL01_POWER215) && (SetSpeed >= (-1*(int32_t)PMSMFOCSL01_POWER215)))
	 {
	   TempSpeed=((SetSpeed*(int32_t)PMSMFOCSL01_POWER215)/HandlePtr->kNormSpeed);
	   HandlePtr->H_Ptr->StartSpeedRef = TempSpeed;
	   Status = (uint32_t)DAVEApp_SUCCESS;
	 }
	 else
	 {
	   Status = (uint32_t)PMSMFOCSL01_INVALID_PARAM;
	   DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
	 }
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  
  return Status;
}
/*
 * This will get the motor end speed reference.
 */
status_t PMSMFOCSL01_GetMotorEndSpeed(const PMSMFOCSL01_HandleType* HandlePtr,
                                            int32_t*                GetSpeed)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    *GetSpeed = ((HandlePtr->H_Ptr->EndSpeedRef*HandlePtr->kNormSpeed) / (int32_t)PMSMFOCSL01_POWER215);
    Status    =  (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This will get the motor start speed reference.
 */
status_t PMSMFOCSL01_GetMotorStartSpeed(const PMSMFOCSL01_HandleType* HandlePtr,
                                              int32_t*                GetSpeed)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    *GetSpeed = ((HandlePtr->H_Ptr->StartSpeedRef*HandlePtr->kNormSpeed) / (int32_t)PMSMFOCSL01_POWER215);
    Status    = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function will give the motor speed
 */
status_t PMSMFOCSL01_GetMotSpeed(const PMSMFOCSL01_HandleType* HandlePtr,
                                       int32_t*                SpeedPtr)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    *SpeedPtr = (((HandlePtr->H_Ptr->MotorSpeed) * 
                  (HandlePtr->kNormSpeed))/(int32_t)PMSMFOCSL01_POWER215);
    Status    = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This will get the status of motor
 */
status_t PMSMFOCSL01_GetMotStatus(const PMSMFOCSL01_HandleType* HandlePtr,
                                        uint16_t*               StatusPtr)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  

  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    *StatusPtr = (uint16_t)HandlePtr->H_Ptr->MotorStatus;
    Status     = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }

  
  return Status;
}
/*
 * This function will return the slew rate
 */
status_t PMSMFOCSL01_GetSlewRate(const PMSMFOCSL01_HandleType* HandlePtr,
                                       uint32_t*               SlewRatePtr)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    if(HandlePtr->H_Ptr->RampCounter==0U)
    {
      HandlePtr->H_Ptr->RampCounter=1U;
     }
    *SlewRatePtr = (uint32_t)(HandlePtr->kNormSlewRate/HandlePtr->H_Ptr->RampCounter);
    Status = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }

  
  return Status;
}
/*
 * This will get the motor direction
 */
status_t PMSMFOCSL01_GetMotDirection(const PMSMFOCSL01_HandleType* HandlePtr,
                                           int32_t*                DirPtr)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    *DirPtr = (int32_t)HandlePtr->H_Ptr->MotorDirection;
    Status = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This will set the motor direction
 */
status_t PMSMFOCSL01_SetMotDirection(const PMSMFOCSL01_HandleType* HandlePtr,
                                           int32_t                 MotDirVal)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    if(PMSMFOCSL01_MOTOR_DIR_FORWARD == MotDirVal)
    {
      if(HandlePtr->H_Ptr->StartSpeedRef < 0)
      {
        HandlePtr->H_Ptr->StartSpeedRef = 
        PMSMFOCSL01_SIGNED_ONE * HandlePtr->H_Ptr->StartSpeedRef;
      }
      if(HandlePtr->H_Ptr->EndSpeedRef < 0)
      {
        HandlePtr->H_Ptr->EndSpeedRef = 
        PMSMFOCSL01_SIGNED_ONE * HandlePtr->H_Ptr->EndSpeedRef;
      }
      HandlePtr->H_Ptr->MotorDirection = (int16_t) MotDirVal;
      Status = (uint32_t)DAVEApp_SUCCESS;
    }
    if (PMSMFOCSL01_MOTOR_DIR_REVERSE == MotDirVal)
    {
      if(HandlePtr->H_Ptr->StartSpeedRef > 0)
      {
        HandlePtr->H_Ptr->StartSpeedRef = 
        PMSMFOCSL01_SIGNED_ONE * HandlePtr->H_Ptr->StartSpeedRef;
      }
      if(HandlePtr->H_Ptr->EndSpeedRef> 0)
      {
        HandlePtr->H_Ptr->EndSpeedRef =
        PMSMFOCSL01_SIGNED_ONE * HandlePtr->H_Ptr->EndSpeedRef;
      }
      HandlePtr->H_Ptr->MotorDirection = (int16_t)MotDirVal;
      Status = (uint32_t)DAVEApp_SUCCESS;
    }
    Status = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}

/*
 * This function sets the slew rate of the motor which dictates the ramp up time of the motor.
 */
status_t PMSMFOCSL01_SetSlewRate(const PMSMFOCSL01_HandleType*  HandlePtr,
                                       uint32_t                 SlewRateVal)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    if(SlewRateVal>=1U)
    {
      HandlePtr->H_Ptr->RampCounter   = (uint32_t)(HandlePtr->kNormSlewRate/SlewRateVal);
      Status = (uint32_t)DAVEApp_SUCCESS;
      if(HandlePtr->H_Ptr->RampCounter<1U)
    	  HandlePtr->H_Ptr->RampCounter =1U;
    }
    else
    {
      Status = (uint32_t)PMSMFOCSL01_INVALID_PARAM;
      DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
    }

  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function sets the speed proportional gain of motor.
 */
status_t PMSMFOCSL01_SetSpeedProportionalGain
(const PMSMFOCSL01_HandleType* HandlePtr,
       uint16_t                Kp)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  
  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED) {
    if(Kp > (uint32_t)PMSMFOCSL01_POWER215) {
      Status = (uint32_t)PMSMFOCSL01_INVALID_PARAM;
      DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
    } else {
      HandlePtr->SpeedPIPtr->Kp = (uint16_t)Kp;
      Status = (uint32_t)DAVEApp_SUCCESS;
    }
  } else  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function sets the speed integral gain of motor.
 */
status_t PMSMFOCSL01_SetSpeedIntegralGain
(const PMSMFOCSL01_HandleType* HandlePtr,
       uint16_t                Ki)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {  
    if(Ki > (uint32_t)PMSMFOCSL01_POWER215) {
      Status = (uint32_t)PMSMFOCSL01_INVALID_PARAM;
      DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
    } else {
      HandlePtr->SpeedPIPtr->Ki   = (uint16_t)Ki;
      Status = (uint32_t)DAVEApp_SUCCESS;
    }
  } else {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function sets the quadrature axis current component(Iq) proportional
 * gain of motor.
 */
status_t PMSMFOCSL01_SetIqProportionalGain
(const PMSMFOCSL01_HandleType* HandlePtr,
       uint16_t                Kp)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED) {  
     if(Kp > (uint32_t)PMSMFOCSL01_POWER215) {
       Status = (uint32_t)PMSMFOCSL01_INVALID_PARAM;
       DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
     } else {
       HandlePtr->IqPIPtr->Kp   = (uint16_t)Kp;
       Status = (uint32_t)DAVEApp_SUCCESS;
     }
  } else  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function sets the quadrature axis current component(Iq) integral
 * gain of motor.
 */
status_t PMSMFOCSL01_SetIqIntegralGain(const PMSMFOCSL01_HandleType* HandlePtr,
                                             uint16_t                Ki)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {  
    if(Ki > (uint32_t)PMSMFOCSL01_POWER215) {
      Status = (uint32_t)PMSMFOCSL01_INVALID_PARAM;
      DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
    } else   {
      HandlePtr->IqPIPtr->Ki   = (uint16_t)Ki;
      Status = (uint32_t)DAVEApp_SUCCESS;
    }
  } else {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function sets the direct axis current component(Iq) proportional gain of
 * motor.
 */
status_t PMSMFOCSL01_SetIdProportionalGain
(const PMSMFOCSL01_HandleType* HandlePtr,
       uint16_t                Kp)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {  
    if(Kp > (uint32_t)PMSMFOCSL01_POWER215) {
      Status = (uint32_t)PMSMFOCSL01_INVALID_PARAM;
      DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
    } else {
      HandlePtr->IdPIPtr->Kp   = (uint16_t)Kp;
      Status = (uint32_t)DAVEApp_SUCCESS;
    }
  } else  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function sets the direct axis current component(Iq) integral gain of 
 * motor.
 */
status_t PMSMFOCSL01_SetIdIntegralGain(const PMSMFOCSL01_HandleType* HandlePtr,
                                             uint16_t                Ki)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    if(Ki > (uint32_t)PMSMFOCSL01_POWER215)
    {
      Status = (uint32_t)PMSMFOCSL01_INVALID_PARAM;
      DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
    } else  {
      HandlePtr->IdPIPtr->Ki   = (uint16_t)Ki;
      Status = (uint32_t)DAVEApp_SUCCESS;
    }
  } else  {
  
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function gets the speed proportional gain of motor.
 */
status_t PMSMFOCSL01_GetSpeedProportionalGain
(const PMSMFOCSL01_HandleType* HandlePtr,
       uint16_t*               Kp)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    *Kp = (uint16_t)HandlePtr->SpeedPIPtr->Kp;
    Status = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function gets the speed integral gain of motor.
 */
status_t PMSMFOCSL01_GetSpeedIntegralGain
(const PMSMFOCSL01_HandleType* HandlePtr,uint16_t *Ki)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    *Ki = (uint16_t)HandlePtr->SpeedPIPtr->Ki;
    Status = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function gets the quadrature axis current component(Iq) proportional 
 * gain of motor.
 */
status_t PMSMFOCSL01_GetIqProportionalGain
(const PMSMFOCSL01_HandleType* HandlePtr,uint16_t *Kp)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    *Kp    = (uint16_t)HandlePtr->IqPIPtr->Kp;
    Status = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function gets the quadrature axis current component(Iq) integral
 * gain of motor.
 */
status_t PMSMFOCSL01_GetIqIntegralGain(const PMSMFOCSL01_HandleType* HandlePtr,
                                             uint16_t*               Ki)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    *Ki    = (uint16_t)HandlePtr->IqPIPtr->Ki;
    Status = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function gets the direct axis current component(Iq) proportional
 * gain of motor.
 */
status_t PMSMFOCSL01_GetIdProportionalGain
(const PMSMFOCSL01_HandleType* HandlePtr,uint16_t *Kp)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    *Kp = (uint16_t)HandlePtr->IdPIPtr->Kp;
    Status = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This function gets the direct axis current component(Iq) integral gain of
 * motor.
 */
status_t PMSMFOCSL01_GetIdIntegralGain(const PMSMFOCSL01_HandleType* HandlePtr,
                                             uint16_t*               Ki)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;

  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    *Ki    = (uint16_t)HandlePtr->IdPIPtr->Ki;
    Status = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*
 * This will set the motor start threshold speed reference.
 */
status_t PMSMFOCSL01_SetMotorSpeedThreshold
(const PMSMFOCSL01_HandleType* HandlePtr,int32_t SetSpeed)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  int32_t  TempSpeed;
  
   if (HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
   {
 	 if((SetSpeed <= (int32_t)PMSMFOCSL01_POWER215) && (SetSpeed >= (-1*(int32_t)PMSMFOCSL01_POWER215)))
 	 {
 	   TempSpeed=((SetSpeed*(int32_t)PMSMFOCSL01_POWER215)/HandlePtr->kNormSpeed);
 	   HandlePtr->H_Ptr->SpeedThreshold = TempSpeed;
 	   Status = (uint32_t)DAVEApp_SUCCESS;
 	 }
 	 else
 	 {
 	   Status = (uint32_t)PMSMFOCSL01_INVALID_PARAM;
 	   DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
 	 }
   }
   else
   {
     DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
   }
   
  return Status;
}
/*
 * This will get the motor start threshold speed reference.
 */
status_t PMSMFOCSL01_GetMotorSpeedThreshold
(const PMSMFOCSL01_HandleType* HandlePtr,int32_t *GetSpeed)
{
  status_t Status = (uint32_t)PMSMFOCSL01_OPERATION_NOT_ALLOWED;
  
  if(HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)
  {
    *GetSpeed = ((HandlePtr->H_Ptr->SpeedThreshold * (int32_t)HandlePtr->kNormSpeed) / (int32_t)PMSMFOCSL01_POWER215);
    Status    = (uint32_t)DAVEApp_SUCCESS;
  }
  else
  {
    DBG002_ERROR(APP_GID, DBG002_MESSAGEID_LITERAL, sizeof(Status), &Status);
  }
  
  return Status;
}
/*******************************************************************************
                              ALGORITHM RELATED FUNCTIONS
*******************************************************************************/
/*
 * This function control motor speed based on the pot value..
 */
void PMSMFOCSL01_PotMeasurement(const PMSMFOCSL01_HandleType* HandlePtr)
{
  uint16_t SpeedValue;
  uint32_t PotValue;
  ADCCH001_HandleType * ChPtrC =
      (ADCCH001_HandleType *)HandlePtr->ADCReqSrcPtrC->kChCHandlerPtr;
  

  /*Read the pot result in PotValue register*/
  PotValue = (uint32_t)(ChPtrC->ADCGrPtr->RES[ChPtrC->kStartResultRegNo] & 
             VADC_G_RES_RESULT_Msk);

  SpeedValue = (uint16_t)(((uint32_t)PotValue*HandlePtr->kPotScaleVal) >> 
               PMSMFOCSL01_POTSCALE);

  if(HandlePtr->H_Ptr->MotorDirection < 0) /*reverse direction*/
  {
    HandlePtr->H_Ptr->EndSpeedRef =  
    (int32_t)(PMSMFOCSL01_SIGNED_ONE * (int32_t)SpeedValue);
  }
  else                               /*Forward direction*/
  {
    HandlePtr->H_Ptr->EndSpeedRef = (int32_t)SpeedValue;
  }
  
}
/*
 * This function calculates the speed ramp up value for algorithm.
 */
void PMSMFOCSL01_SpeedRampup(const PMSMFOCSL01_HandleType* HandlePtr)
{
  uint32_t TempSpeedRef;
  
  

  if (HandlePtr->H_Ptr->State != PMSMFOCSL01_UNINITIALIZED)  {
      /*Update the speed reference value.*/
      if (HandlePtr->H_Ptr->RampScheduler > HandlePtr->H_Ptr->RampCounter) {
          if (PMSMFOCSL01_OPENLOOP == HandlePtr->H_Ptr->State) {
              if ((HandlePtr->H_Ptr->EndSpeedRef > 0) && 
                  (HandlePtr->H_Ptr->SpeedRef < HandlePtr->H_Ptr->SpeedThreshold)) {
                  HandlePtr->H_Ptr->SpeedRef++;
                  SET_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_RAMP_Pos);
              } else if ((HandlePtr->H_Ptr->EndSpeedRef < 0) && 
                  (HandlePtr->H_Ptr->SpeedRef > HandlePtr->H_Ptr->SpeedThreshold)) {
                  HandlePtr->H_Ptr->SpeedRef--;
                  SET_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_RAMP_Pos);
              } else {
                  CLR_BIT(HandlePtr->H_Ptr->MotorStatus,PMSMFOCSL01_OL_Pos);
                  SET_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_FOC_Pos);
                  TempSpeedRef = ((uint32_t)HandlePtr->H_Ptr->SpeedRef<<16U);                  
                  HandlePtr->SpeedPT1Ptr->PT1buf     = (int32_t)TempSpeedRef;
                  HandlePtr->H_Ptr->State            = PMSMFOCSL01_RUNNING;
                  HandlePtr->H_Ptr->SpeedControlRate = 0U;
              }
          } else {
              if (HandlePtr->H_Ptr->EndSpeedRef > HandlePtr->H_Ptr->SpeedRef) {
                  HandlePtr->H_Ptr->SpeedRef++;
                  SET_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_RAMP_Pos);
              } else if(HandlePtr->H_Ptr->EndSpeedRef < HandlePtr->H_Ptr->SpeedRef) {
                  HandlePtr->H_Ptr->SpeedRef--;
                  SET_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_RAMP_Pos);
              } else {
                  CLR_BIT(HandlePtr->H_Ptr->MotorStatus, PMSMFOCSL01_RAMP_Pos);
              }
          }
          HandlePtr->H_Ptr->RampScheduler = 0U;
      }
      HandlePtr->H_Ptr->RampScheduler++;
  } 
  
}
/*
 * This function will read the dc link value and do the voltage compensation.
 */
void PMSMFOCSL01_VoltageCompensation(const PMSMFOCSL01_HandleType* HandlePtr)
{
  ADCCH001_HandleType * ChPtrB =
 (ADCCH001_HandleType *)HandlePtr->ADCReqSrcPtrB->kChBHandlerPtr;

  

  /*Read the dc link voltage*/
  HandlePtr->H_Ptr->ActVolt =
  (uint32_t)(ChPtrB->ADCGrPtr->RES[ChPtrB->kStartResultRegNo] & VADC_G_RES_RESULT_Msk);
  MOTORLIBS_PT1Controller(HandlePtr->VoltCompPT1Ptr,
                          (int32_t)HandlePtr->H_Ptr->ActVolt);

  /*Do the voltage compensation based on dc link value*/
  HandlePtr->H_Ptr->Amplitude =
 (uint32_t)(((uint32_t)HandlePtr->H_Ptr->Amplitude * HandlePtr->kSpecifiedVolt) /
 ((uint16_t)HandlePtr->VoltCompPT1Ptr->PT1OutVal));

  
}
/*
 * This function Read the DC link current values from ADC.
 */
void PMSMFOCSL01_ReadCurrent(const PMSMFOCSL01_HandleType* HandlePtr)
{
  ADCCH001_HandleType *ChPtrA = 
  (ADCCH001_HandleType*)HandlePtr->ADCReqSrcPtrA->kChAHandlerPtr;

  /* Standard SVM - 7 Segmen Symmetrical PWM, Read 4 ADC values from FIFO*/
  if (PMSMFOCSL01_FALSE == HandlePtr->SVMAppPtr->kSVMConfig)
  {
    HandlePtr->H_Ptr->NegativeCurrent  = 
    (ChPtrA->ADCGrPtr->RES[(ChPtrA->kEndResultRegNo)] & VADC_G_RES_RESULT_Msk);
    HandlePtr->H_Ptr->PositiveCurrent  =
    (ChPtrA->ADCGrPtr->RES[(ChPtrA->kEndResultRegNo)] & VADC_G_RES_RESULT_Msk);
    HandlePtr->H_Ptr->PositiveCurrent +=
    (ChPtrA->ADCGrPtr->RES[(ChPtrA->kEndResultRegNo)] & VADC_G_RES_RESULT_Msk)-(HandlePtr->H_Ptr->AmpOffset*2U);
    HandlePtr->H_Ptr->NegativeCurrent += 
    (ChPtrA->ADCGrPtr->RES[(ChPtrA->kEndResultRegNo)] & VADC_G_RES_RESULT_Msk)-(HandlePtr->H_Ptr->AmpOffset*2U);
  }
  /*Pseudo Zero Vector - 3 and 4 Active Vectors */
  else if((PMSMFOCSL01_PSEUDO3V == HandlePtr->SVMAppPtr->kSVMConfig) ||
          (PMSMFOCSL01_PSEDO4V  == HandlePtr->SVMAppPtr->kSVMConfig))
  {
    HandlePtr->H_Ptr->NegativeCurrent = 
    ((ChPtrA->ADCGrPtr->RES[(ChPtrA->kEndResultRegNo)] & VADC_G_RES_RESULT_Msk)-HandlePtr->H_Ptr->AmpOffset)<<1U;
    HandlePtr->H_Ptr->PositiveCurrent = 
    ((ChPtrA->ADCGrPtr->RES[(ChPtrA->kEndResultRegNo)] & VADC_G_RES_RESULT_Msk)-HandlePtr->H_Ptr->AmpOffset)<<1U;
  }
  /* Standard SVM - 7 Segment Asymmetrical and 5 Segment*/
  else
  {
    HandlePtr->H_Ptr->PositiveCurrent =
    ((ChPtrA->ADCGrPtr->RES[(ChPtrA->kEndResultRegNo)] & VADC_G_RES_RESULT_Msk)-HandlePtr->H_Ptr->AmpOffset)<<1U;
    HandlePtr->H_Ptr->NegativeCurrent =
    ((ChPtrA->ADCGrPtr->RES[(ChPtrA->kEndResultRegNo)] & VADC_G_RES_RESULT_Msk)-HandlePtr->H_Ptr->AmpOffset)<<1U;
  }
}
/*
 * This function construct phase current from DC link Current Values.
 */
void PMSMFOCSL01_PhaseCurrentCalc(const PMSMFOCSL01_HandleType* HandlePtr)
{
  switch(HandlePtr->H_Ptr->SectorStore)
  {
    case 0:
      HandlePtr->H_Ptr->Ia = HandlePtr->H_Ptr->PositiveCurrent;
      HandlePtr->H_Ptr->Ib = 
      HandlePtr->H_Ptr->NegativeCurrent-HandlePtr->H_Ptr->PositiveCurrent;
      break;
    case 1:
      HandlePtr->H_Ptr->Ia =
      HandlePtr->H_Ptr->NegativeCurrent-HandlePtr->H_Ptr->PositiveCurrent;
      HandlePtr->H_Ptr->Ib =
      HandlePtr->H_Ptr->PositiveCurrent;
      break;
    case 2:
      HandlePtr->H_Ptr->Ia = 
      PMSMFOCSL01_SIGNED_ONE * (int32_t)(HandlePtr->H_Ptr->NegativeCurrent);
      HandlePtr->H_Ptr->Ib =
      HandlePtr->H_Ptr->PositiveCurrent;
      break;
    case 3:
      HandlePtr->H_Ptr->Ia = 
      PMSMFOCSL01_SIGNED_ONE * (int32_t)(HandlePtr->H_Ptr->NegativeCurrent);
      HandlePtr->H_Ptr->Ib =
      HandlePtr->H_Ptr->NegativeCurrent-HandlePtr->H_Ptr->PositiveCurrent;
      break;
    case 4:
      HandlePtr->H_Ptr->Ia =
      HandlePtr->H_Ptr->NegativeCurrent-HandlePtr->H_Ptr->PositiveCurrent;
      HandlePtr->H_Ptr->Ib = 
      PMSMFOCSL01_SIGNED_ONE * (int32_t)(HandlePtr->H_Ptr->NegativeCurrent);
      break;
    case 5:
      HandlePtr->H_Ptr->Ia = HandlePtr->H_Ptr->PositiveCurrent;
      HandlePtr->H_Ptr->Ib = 
      PMSMFOCSL01_SIGNED_ONE * (int32_t)(HandlePtr->H_Ptr->NegativeCurrent);
      break;
    default:
      break;
   }
}
/*
 * This function estimate the flux and calculate the flux angle .
 */
void PMSMFOCSL01_FluxEstimation(const PMSMFOCSL01_HandleType* HandlePtr)
{
  volatile int32_t FluxTemp;
  /* Calculation of Flux Alpha Component*/
  FluxTemp = HandlePtr->H_Ptr->Valpha-((HandlePtr->H_Ptr->Ialpha*(int32_t)HandlePtr->kMotorResistance)>>(PMSMFOCSL01_RESISTANCESCALE-HandlePtr->kResistanceScale));

  MOTORLIBS_PT12Controller(HandlePtr->ValphaPT12Ptr,FluxTemp);

  HandlePtr->H_Ptr->FluxAlpha = 
  HandlePtr->ValphaPT12Ptr->PT1OutVal-((HandlePtr->H_Ptr->Ialpha*(int32_t)HandlePtr->kMotorInductance)>>(PMSMFOCSL01_INDUCTANCESCALE - HandlePtr->kInductanceScale));

  /* Calculation of Flux Beta Component*/
  FluxTemp = HandlePtr->H_Ptr->Vbeta-((HandlePtr->H_Ptr->Ibeta*(int32_t)HandlePtr->kMotorResistance)>>(PMSMFOCSL01_RESISTANCESCALE-HandlePtr->kResistanceScale));

  MOTORLIBS_PT12Controller(HandlePtr->VbetaPT12Ptr,FluxTemp);

  HandlePtr->H_Ptr->FluxBeta = 
  HandlePtr->VbetaPT12Ptr->PT1OutVal-((HandlePtr->H_Ptr->Ibeta*(int32_t)HandlePtr->kMotorInductance)>>(PMSMFOCSL01_INDUCTANCESCALE-HandlePtr->kInductanceScale));

  /* Calculation of Flux Angle*/
  HandlePtr->H_Ptr->FluxAmp = MOTORLIBS_Car2Pol(HandlePtr->H_Ptr->FluxAlpha,HandlePtr->H_Ptr->FluxBeta,&HandlePtr->H_Ptr->FluxAngle);

  //30.10.2016.
  HandlePtr->H_Ptr->EAlpha = HandlePtr->H_Ptr->FluxAlpha*HandlePtr->H_Ptr->Ys;
  HandlePtr->H_Ptr->EBeta = HandlePtr->H_Ptr->FluxBeta*HandlePtr->H_Ptr->Xc;
  HandlePtr->H_Ptr->E = HandlePtr->H_Ptr->EBeta-HandlePtr->H_Ptr->EAlpha;
  MOTORLIBS_PIController(HandlePtr->PLLPIPtr,HandlePtr->H_Ptr->EAlpha,HandlePtr->H_Ptr->EBeta);
  HandlePtr->H_Ptr->MotorSpeedR = HandlePtr->PLLPIPtr->PiOutVal;
  MOTORLIBS_PT12Controller(HandlePtr->PLLPT12Ptr,HandlePtr->H_Ptr->MotorSpeedR);
  HandlePtr->H_Ptr->FluxAngle1 = HandlePtr->PLLPT12Ptr->PT1OutVal;
  HandlePtr->H_Ptr->Ys =
      MOTORLIBS_Pol2Car(HandlePtr->H_Ptr->FluxAngle1,(0x4DBA),&HandlePtr->H_Ptr->Xc);


  //


}

void PMSMFOCSL01_FOCCalculation(const PMSMFOCSL01_HandleType* HandlePtr)
{
  uint32_t Amplitude,SpeedValue;
  int32_t SpeedRef;
  PMSMFOCSL01_DynamicHandleType* AppHdlPtr =
  (PMSMFOCSL01_DynamicHandleType*)(void*)HandlePtr->H_Ptr;
  
  /* Construct Phase Ia and Ib Current */
  PMSMFOCSL01_PhaseCurrentCalc(HandlePtr);

  /* Calculation of stationary orthogonal reference frame Current Components 
     Ialpha and Ibeta */
  AppHdlPtr->Ibeta = 
  MOTORLIBS_ClarkTransform(AppHdlPtr->Ia,AppHdlPtr->Ib,&(AppHdlPtr->Ialpha));
  
  /* Save the Flux Angle value into a buffer used for Speed calculation*/
  if(AppHdlPtr->SpeedControlRate == 0U)
  {
    AppHdlPtr->FluxAngleStore = AppHdlPtr->FluxAngle;
  }
  
  /* Calculation of Flux and Flux Angle*/
  PMSMFOCSL01_FluxEstimation(HandlePtr);
  
  /* Calculation of Id and Iq Current Components*/
  AppHdlPtr->Id = 
  MOTORLIBS_ParkTransform(AppHdlPtr->Ialpha,AppHdlPtr->Ibeta,AppHdlPtr->FluxAngle,&AppHdlPtr->Iq);

  if(AppHdlPtr->State == PMSMFOCSL01_OPENLOOP)
  {
    /* Constant V/F Open loop - Angle calculation*/
    AppHdlPtr->SpeedControlRate  += HandlePtr->kCurrentControlRate;
    if((AppHdlPtr->SpeedControlRate) >= HandlePtr->kSpeedControlRate)
    {
      AppHdlPtr->Angle            += (int16_t)AppHdlPtr->SpeedRef;
      MOTORLIBS_PT1Controller(HandlePtr->SpeedPT1Ptr,
                              (int16_t)(AppHdlPtr->FluxAngle-AppHdlPtr->FluxAngleStore));
      AppHdlPtr->MotorSpeed        = HandlePtr->SpeedPT1Ptr->PT1OutVal;
      AppHdlPtr->SpeedControlRate  = 0U;
    }
    
    /* Constant V/F Open loop - Voltage Amplitude calculation*/
    SpeedRef =HandlePtr->H_Ptr->SpeedRef;
    if((SpeedRef) < 0)
    {
      SpeedRef = PMSMFOCSL01_SIGNED_ONE *SpeedRef;
    }
    SpeedValue = (uint32_t)SpeedRef;
    Amplitude  = (uint32_t)(((SpeedValue * HandlePtr->StartupHPtr->VFConst) >>
                 PMSMFOCSL01_VFSCALE));

    if(Amplitude < (HandlePtr->StartupHPtr->VFOffset))
    {
      Amplitude = HandlePtr->StartupHPtr->VFOffset;
    }
    AppHdlPtr->Vbeta = 
    MOTORLIBS_Pol2Car(AppHdlPtr->Angle,Amplitude,&AppHdlPtr->Valpha);
  }
  /* If FOC is enabled*/
  else
  {
    /* Calculation of Motor Speed value using Flux Angle*/
    AppHdlPtr->SpeedControlRate += HandlePtr->kCurrentControlRate;
    if (AppHdlPtr->SpeedControlRate >= HandlePtr->kSpeedControlRate)
    {
       MOTORLIBS_PT1Controller(HandlePtr->SpeedPT1Ptr,
                               (int16_t)(AppHdlPtr->FluxAngle-AppHdlPtr->FluxAngleStore));

       AppHdlPtr->MotorSpeed = HandlePtr->SpeedPT1Ptr->PT1OutVal;

       AppHdlPtr->SpeedControlRate = 0U;
    }
    /*Speed PI Control to calculate Iq reference value*/
    MOTORLIBS_PIController(HandlePtr->SpeedPIPtr,
                           AppHdlPtr->SpeedRef,
                           AppHdlPtr->MotorSpeed);

    /*Current PI Control for finding Vq*/
    MOTORLIBS_PIController(HandlePtr->IqPIPtr,
                           (int32_t)HandlePtr->SpeedPIPtr->PiOutVal,
                           AppHdlPtr->Iq);

    AppHdlPtr->Vq = HandlePtr->IqPIPtr->PiOutVal;

    /*Current PI Control for finding Vd*/
    MOTORLIBS_PIController(HandlePtr->IdPIPtr,AppHdlPtr->IdRef,AppHdlPtr->Id);

    AppHdlPtr->Vd = HandlePtr->IdPIPtr->PiOutVal;

    /*Inverse park transform to calculate Valpha & Vbeta*/
    AppHdlPtr->Valpha = MOTORLIBS_IParkTransform( AppHdlPtr->Vd,
                                                  AppHdlPtr->Vq,
                                                  AppHdlPtr->FluxAngle,
                                                 &AppHdlPtr->Vbeta);

    /*Cartesian to Polar Transform to calculate Voltage Amplitude and Angle*/
    Amplitude = MOTORLIBS_Car2Pol( AppHdlPtr->Valpha,
                                   AppHdlPtr->Vbeta,
                                  &AppHdlPtr->Angle);
  }

  if( Amplitude > HandlePtr->kAmplitudeMax)
  {
    Amplitude = HandlePtr->kAmplitudeMax;
  }

  /* Amplitude calculation*/
  AppHdlPtr->Amplitude = 
  (HandlePtr->kAmplitudeScale*Amplitude) >> PMSMFOCSL01_AMPSCALE;
}

/**
* @endcond
*/
/*CODE_BLOCK_END*/

